"use strict";
/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectControls = void 0;
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var ObjectControls;
(function (ObjectControls) {
    function mouseOnScreen(out, page, viewport) {
        return linear_algebra_1.Vec2.set(out, (page[0] - viewport.x) / viewport.width, (page[1] - viewport.y) / viewport.height);
    }
    var panMouseChange = linear_algebra_1.Vec2();
    var panObjUp = linear_algebra_1.Vec3();
    var panOffset = linear_algebra_1.Vec3();
    var eye = linear_algebra_1.Vec3();
    var panStart = linear_algebra_1.Vec2();
    var panEnd = linear_algebra_1.Vec2();
    var target = linear_algebra_1.Vec3();
    /**
     * Get vector for movement in camera projection plane:
     * `pageStart` and `pageEnd` are 2d window coordinates;
     * `ref` defines the plane depth, if not given `camera.target` is used
     */
    function panDirection(out, pageStart, pageEnd, camera, ref) {
        mouseOnScreen(panStart, pageStart, camera.viewport);
        mouseOnScreen(panEnd, pageEnd, camera.viewport);
        linear_algebra_1.Vec2.sub(panMouseChange, linear_algebra_1.Vec2.copy(panMouseChange, panEnd), panStart);
        linear_algebra_1.Vec3.sub(eye, camera.position, camera.target);
        if (!ref || camera.state.mode === 'orthographic')
            linear_algebra_1.Vec3.copy(target, camera.target);
        else
            linear_algebra_1.Vec3.projectPointOnVector(target, ref, eye, camera.position);
        var dist = linear_algebra_1.Vec3.distance(camera.position, target);
        var height = 2 * Math.tan(camera.state.fov / 2) * dist;
        var zoom = camera.viewport.height / height;
        panMouseChange[0] *= (1 / zoom) * camera.viewport.width * camera.pixelRatio;
        panMouseChange[1] *= (1 / zoom) * camera.viewport.height * camera.pixelRatio;
        linear_algebra_1.Vec3.cross(panOffset, linear_algebra_1.Vec3.copy(panOffset, eye), camera.up);
        linear_algebra_1.Vec3.setMagnitude(panOffset, panOffset, panMouseChange[0]);
        linear_algebra_1.Vec3.setMagnitude(panObjUp, camera.up, panMouseChange[1]);
        linear_algebra_1.Vec3.add(panOffset, panOffset, panObjUp);
        return linear_algebra_1.Vec3.negate(out, panOffset);
    }
    ObjectControls.panDirection = panDirection;
})(ObjectControls = exports.ObjectControls || (exports.ObjectControls = {}));
//# sourceMappingURL=object.js.map