"use strict";
/**
 * Copyright (c) 2017-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.intersectionSize = exports.intersect = exports.findRange = exports.findPredecessorIndexInInterval = exports.findPredecessorIndex = exports.isSubInterval = exports.areIntersecting = exports.areEqual = exports.getAt = exports.indexOf = exports.has = exports.toString = exports.hashCode = exports.size = exports.max = exports.min = exports.end = exports.start = exports.is = exports.ofLength = exports.ofBounds = exports.ofRange = exports.Empty = void 0;
var tslib_1 = require("tslib");
var tuple_1 = tslib_1.__importDefault(require("../tuple"));
exports.Empty = tuple_1.default.Zero;
function ofRange(min, max) { return max < min ? tuple_1.default.create(min, min) : tuple_1.default.create(min, max + 1); }
exports.ofRange = ofRange;
function ofBounds(start, end) { return end <= start ? tuple_1.default.create(start, start) : tuple_1.default.create(start, end); }
exports.ofBounds = ofBounds;
function ofLength(length) { return length < 0 ? tuple_1.default.create(0, 0) : tuple_1.default.create(0, length); }
exports.ofLength = ofLength;
exports.is = tuple_1.default.is;
exports.start = tuple_1.default.fst;
exports.end = tuple_1.default.snd;
exports.min = tuple_1.default.fst;
function max(i) { return tuple_1.default.snd(i) - 1; }
exports.max = max;
function size(i) { return tuple_1.default.snd(i) - tuple_1.default.fst(i); }
exports.size = size;
exports.hashCode = tuple_1.default.hashCode;
exports.toString = tuple_1.default.toString;
function has(int, v) { return tuple_1.default.fst(int) <= v && v < tuple_1.default.snd(int); }
exports.has = has;
/** Returns the index of `x` in `set` or -1 if not found. */
function indexOf(int, x) { var m = exports.start(int); return x >= m && x < exports.end(int) ? x - m : -1; }
exports.indexOf = indexOf;
function getAt(int, i) { return tuple_1.default.fst(int) + i; }
exports.getAt = getAt;
exports.areEqual = tuple_1.default.areEqual;
function areIntersecting(a, b) {
    var sa = size(a), sb = size(b);
    if (sa === 0 && sb === 0)
        return true;
    return sa > 0 && sb > 0 && max(a) >= exports.min(b) && exports.min(a) <= max(b);
}
exports.areIntersecting = areIntersecting;
function isSubInterval(a, b) {
    if (!size(a))
        return size(b) === 0;
    if (!size(b))
        return true;
    return exports.start(a) <= exports.start(b) && exports.end(a) >= exports.end(b);
}
exports.isSubInterval = isSubInterval;
function findPredecessorIndex(int, v) {
    var s = exports.start(int);
    if (v <= s)
        return 0;
    var e = exports.end(int);
    if (v >= e)
        return e - s;
    return v - s;
}
exports.findPredecessorIndex = findPredecessorIndex;
function findPredecessorIndexInInterval(int, v, bounds) {
    var bS = exports.start(bounds);
    var s = exports.start(int);
    if (v <= bS + s)
        return bS;
    var bE = exports.end(bounds);
    if (v >= bE + s)
        return bE;
    return v - s;
}
exports.findPredecessorIndexInInterval = findPredecessorIndexInInterval;
function findRange(int, min, max) {
    return ofBounds(findPredecessorIndex(int, min), findPredecessorIndex(int, max + 1));
}
exports.findRange = findRange;
function intersect(a, b) {
    if (!areIntersecting(a, b))
        return exports.Empty;
    return ofBounds(Math.max(exports.start(a), exports.start(b)), Math.min(exports.end(a), exports.end(b)));
}
exports.intersect = intersect;
function intersectionSize(a, b) {
    return size(findRange(a, exports.min(b), max(b)));
}
exports.intersectionSize = intersectionSize;
//# sourceMappingURL=interval.js.map