"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DirectVolumeRenderable = exports.DirectVolumeSchema = void 0;
var tslib_1 = require("tslib");
var renderable_1 = require("../renderable");
var render_item_1 = require("../webgl/render-item");
var schema_1 = require("./schema");
var shader_code_1 = require("../shader-code");
var mol_util_1 = require("../../mol-util");
exports.DirectVolumeSchema = {
    uColor: schema_1.UniformSpec('v3'),
    uColorTexDim: schema_1.UniformSpec('v2'),
    tColor: schema_1.TextureSpec('image-uint8', 'rgb', 'ubyte', 'nearest'),
    dColorType: schema_1.DefineSpec('string', ['uniform', 'attribute', 'instance', 'group', 'groupInstance', 'vertex', 'vertexInstance']),
    uMarkerTexDim: schema_1.UniformSpec('v2'),
    tMarker: schema_1.TextureSpec('image-uint8', 'alpha', 'ubyte', 'nearest'),
    uOverpaintTexDim: schema_1.UniformSpec('v2'),
    tOverpaint: schema_1.TextureSpec('image-uint8', 'rgba', 'ubyte', 'nearest'),
    dOverpaint: schema_1.DefineSpec('boolean'),
    uTransparencyTexDim: schema_1.UniformSpec('v2'),
    tTransparency: schema_1.TextureSpec('image-uint8', 'alpha', 'ubyte', 'nearest'),
    dTransparency: schema_1.DefineSpec('boolean'),
    transparencyAverage: schema_1.ValueSpec('number'),
    dClipObjectCount: schema_1.DefineSpec('number'),
    dClipVariant: schema_1.DefineSpec('string', ['instance', 'pixel']),
    uClippingTexDim: schema_1.UniformSpec('v2'),
    tClipping: schema_1.TextureSpec('image-uint8', 'alpha', 'ubyte', 'nearest'),
    dClipping: schema_1.DefineSpec('boolean'),
    uVertexCount: schema_1.UniformSpec('i'),
    uInstanceCount: schema_1.UniformSpec('i'),
    uGroupCount: schema_1.UniformSpec('i'),
    uInvariantBoundingSphere: schema_1.UniformSpec('v4'),
    aInstance: schema_1.AttributeSpec('float32', 1, 1),
    aTransform: schema_1.AttributeSpec('float32', 16, 1),
    drawCount: schema_1.ValueSpec('number'),
    instanceCount: schema_1.ValueSpec('number'),
    alpha: schema_1.ValueSpec('number'),
    matrix: schema_1.ValueSpec('m4'),
    transform: schema_1.ValueSpec('float32'),
    extraTransform: schema_1.ValueSpec('float32'),
    hasReflection: schema_1.ValueSpec('boolean'),
    boundingSphere: schema_1.ValueSpec('sphere'),
    invariantBoundingSphere: schema_1.ValueSpec('sphere'),
    aPosition: schema_1.AttributeSpec('float32', 3, 0),
    elements: schema_1.ElementsSpec('uint32'),
    uAlpha: schema_1.UniformSpec('f'),
    uIsoValue: schema_1.UniformSpec('v2'),
    uBboxMin: schema_1.UniformSpec('v3'),
    uBboxMax: schema_1.UniformSpec('v3'),
    uBboxSize: schema_1.UniformSpec('v3'),
    uMaxSteps: schema_1.UniformSpec('i'),
    uStepScale: schema_1.UniformSpec('f'),
    uJumpLength: schema_1.UniformSpec('f'),
    uTransform: schema_1.UniformSpec('m4'),
    uGridDim: schema_1.UniformSpec('v3'),
    dRenderMode: schema_1.DefineSpec('string', ['isosurface', 'volume']),
    dSingleLayer: schema_1.DefineSpec('boolean'),
    tTransferTex: schema_1.TextureSpec('image-uint8', 'rgba', 'ubyte', 'linear'),
    uTransferScale: schema_1.UniformSpec('f'),
    dGridTexType: schema_1.DefineSpec('string', ['2d', '3d']),
    uGridTexDim: schema_1.UniformSpec('v3'),
    tGridTex: schema_1.TextureSpec('texture', 'rgba', 'ubyte', 'linear'),
    uGridStats: schema_1.UniformSpec('v4'),
    uCellDim: schema_1.UniformSpec('v3'),
    uCartnToUnit: schema_1.UniformSpec('m4'),
    uUnitToCartn: schema_1.UniformSpec('m4'),
    dPackedGroup: schema_1.DefineSpec('boolean'),
    dDoubleSided: schema_1.DefineSpec('boolean'),
    dFlipSided: schema_1.DefineSpec('boolean'),
    dFlatShaded: schema_1.DefineSpec('boolean'),
    dIgnoreLight: schema_1.DefineSpec('boolean'),
    dXrayShaded: schema_1.DefineSpec('boolean'),
};
function DirectVolumeRenderable(ctx, id, values, state, materialId) {
    var schema = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, schema_1.GlobalUniformSchema), schema_1.GlobalTextureSchema), schema_1.InternalSchema), exports.DirectVolumeSchema);
    if (!ctx.isWebGL2) {
        // workaround for webgl1 limitation that loop counters need to be `const`
        schema.uMaxSteps = schema_1.DefineSpec('number');
    }
    var internalValues = {
        uObjectId: mol_util_1.ValueCell.create(id),
    };
    var shaderCode = shader_code_1.DirectVolumeShaderCode;
    var renderItem = render_item_1.createGraphicsRenderItem(ctx, 'triangles', shaderCode, schema, tslib_1.__assign(tslib_1.__assign({}, values), internalValues), materialId);
    return renderable_1.createRenderable(renderItem, values, state);
}
exports.DirectVolumeRenderable = DirectVolumeRenderable;
//# sourceMappingURL=direct-volume.js.map