"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.addShaderDefines = exports.ImageShaderCode = exports.DirectVolumeShaderCode = exports.MeshShaderCode = exports.LinesShaderCode = exports.TextShaderCode = exports.SpheresShaderCode = exports.PointsShaderCode = exports.ShaderCode = void 0;
var tslib_1 = require("tslib");
var id_factory_1 = require("../mol-util/id-factory");
var compat_1 = require("./webgl/compat");
var shaderCodeId = id_factory_1.idFactory();
var apply_fog_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/apply-fog.glsl"));
var apply_interior_color_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/apply-interior-color.glsl"));
var apply_light_color_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/apply-light-color.glsl"));
var apply_marker_color_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/apply-marker-color.glsl"));
var assign_clipping_varying_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/assign-clipping-varying.glsl"));
var assign_color_varying_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/assign-color-varying.glsl"));
var assign_group_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/assign-group.glsl"));
var assign_marker_varying_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/assign-marker-varying.glsl"));
var assign_material_color_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/assign-material-color.glsl"));
var assign_position_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/assign-position.glsl"));
var assign_size_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/assign-size.glsl"));
var check_picking_alpha_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/check-picking-alpha.glsl"));
var clip_instance_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/clip-instance.glsl"));
var clip_pixel_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/clip-pixel.glsl"));
var color_frag_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/color-frag-params.glsl"));
var color_vert_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/color-vert-params.glsl"));
var common_clip_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/common-clip.glsl"));
var common_frag_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/common-frag-params.glsl"));
var common_vert_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/common-vert-params.glsl"));
var common_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/common.glsl"));
var light_frag_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/light-frag-params.glsl"));
var matrix_scale_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/matrix-scale.glsl"));
var normal_frag_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/normal-frag-params.glsl"));
var read_from_texture_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/read-from-texture.glsl"));
var size_vert_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/size-vert-params.glsl"));
var texture3d_from_1d_trilinear_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/texture3d-from-1d-trilinear.glsl"));
var texture3d_from_2d_linear_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/texture3d-from-2d-linear.glsl"));
var texture3d_from_2d_nearest_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/texture3d-from-2d-nearest.glsl"));
var wboit_params_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/wboit-params.glsl"));
var wboit_write_glsl_1 = tslib_1.__importDefault(require("./shader/chunks/wboit-write.glsl"));
var ShaderChunks = {
    apply_fog: apply_fog_glsl_1.default,
    apply_interior_color: apply_interior_color_glsl_1.default,
    apply_light_color: apply_light_color_glsl_1.default,
    apply_marker_color: apply_marker_color_glsl_1.default,
    assign_clipping_varying: assign_clipping_varying_glsl_1.default,
    assign_color_varying: assign_color_varying_glsl_1.default,
    assign_group: assign_group_glsl_1.default,
    assign_marker_varying: assign_marker_varying_glsl_1.default,
    assign_material_color: assign_material_color_glsl_1.default,
    assign_position: assign_position_glsl_1.default,
    assign_size: assign_size_glsl_1.default,
    check_picking_alpha: check_picking_alpha_glsl_1.default,
    clip_instance: clip_instance_glsl_1.default,
    clip_pixel: clip_pixel_glsl_1.default,
    color_frag_params: color_frag_params_glsl_1.default,
    color_vert_params: color_vert_params_glsl_1.default,
    common_clip: common_clip_glsl_1.default,
    common_frag_params: common_frag_params_glsl_1.default,
    common_vert_params: common_vert_params_glsl_1.default,
    common: common_glsl_1.default,
    light_frag_params: light_frag_params_glsl_1.default,
    matrix_scale: matrix_scale_glsl_1.default,
    normal_frag_params: normal_frag_params_glsl_1.default,
    read_from_texture: read_from_texture_glsl_1.default,
    size_vert_params: size_vert_params_glsl_1.default,
    texture3d_from_1d_trilinear: texture3d_from_1d_trilinear_glsl_1.default,
    texture3d_from_2d_linear: texture3d_from_2d_linear_glsl_1.default,
    texture3d_from_2d_nearest: texture3d_from_2d_nearest_glsl_1.default,
    wboit_params: wboit_params_glsl_1.default,
    wboit_write: wboit_write_glsl_1.default
};
var reInclude = /^(?!\/\/)\s*#include\s+(\S+)/gmi;
var reSingleLineComment = /[ \t]*\/\/.*\n/g;
var reMultiLineComment = /[ \t]*\/\*[\s\S]*?\*\//g;
var reMultipleLinebreaks = /\n{2,}/g;
function addIncludes(text) {
    return text
        .replace(reInclude, function (_, p1) {
        var chunk = ShaderChunks[p1];
        if (!chunk)
            throw new Error("empty chunk, '" + p1 + "'");
        return chunk;
    })
        .trim()
        .replace(reSingleLineComment, '\n')
        .replace(reMultiLineComment, '\n')
        .replace(reMultipleLinebreaks, '\n');
}
function ShaderCode(name, vert, frag, extensions) {
    if (extensions === void 0) { extensions = {}; }
    return { id: shaderCodeId(), name: name, vert: addIncludes(vert), frag: addIncludes(frag), extensions: extensions };
}
exports.ShaderCode = ShaderCode;
var points_vert_1 = tslib_1.__importDefault(require("./shader/points.vert"));
var points_frag_1 = tslib_1.__importDefault(require("./shader/points.frag"));
exports.PointsShaderCode = ShaderCode('points', points_vert_1.default, points_frag_1.default, { drawBuffers: 'optional' });
var spheres_vert_1 = tslib_1.__importDefault(require("./shader/spheres.vert"));
var spheres_frag_1 = tslib_1.__importDefault(require("./shader/spheres.frag"));
exports.SpheresShaderCode = ShaderCode('spheres', spheres_vert_1.default, spheres_frag_1.default, { fragDepth: 'required', drawBuffers: 'optional' });
var text_vert_1 = tslib_1.__importDefault(require("./shader/text.vert"));
var text_frag_1 = tslib_1.__importDefault(require("./shader/text.frag"));
exports.TextShaderCode = ShaderCode('text', text_vert_1.default, text_frag_1.default, { standardDerivatives: 'required', drawBuffers: 'optional' });
var lines_vert_1 = tslib_1.__importDefault(require("./shader/lines.vert"));
var lines_frag_1 = tslib_1.__importDefault(require("./shader/lines.frag"));
exports.LinesShaderCode = ShaderCode('lines', lines_vert_1.default, lines_frag_1.default, { drawBuffers: 'optional' });
var mesh_vert_1 = tslib_1.__importDefault(require("./shader/mesh.vert"));
var mesh_frag_1 = tslib_1.__importDefault(require("./shader/mesh.frag"));
exports.MeshShaderCode = ShaderCode('mesh', mesh_vert_1.default, mesh_frag_1.default, { standardDerivatives: 'optional', drawBuffers: 'optional' });
var direct_volume_vert_1 = tslib_1.__importDefault(require("./shader/direct-volume.vert"));
var direct_volume_frag_1 = tslib_1.__importDefault(require("./shader/direct-volume.frag"));
exports.DirectVolumeShaderCode = ShaderCode('direct-volume', direct_volume_vert_1.default, direct_volume_frag_1.default, { fragDepth: 'optional', drawBuffers: 'optional' });
var image_vert_1 = tslib_1.__importDefault(require("./shader/image.vert"));
var image_frag_1 = tslib_1.__importDefault(require("./shader/image.frag"));
exports.ImageShaderCode = ShaderCode('image', image_vert_1.default, image_frag_1.default, { drawBuffers: 'optional' });
function getDefinesCode(defines) {
    if (defines === undefined)
        return '';
    var lines = [];
    for (var name_1 in defines) {
        var define = defines[name_1];
        var v = define.ref.value;
        if (v !== undefined) {
            if (typeof v === 'string') {
                lines.push("#define " + name_1 + "_" + v);
            }
            else if (typeof v === 'number') {
                lines.push("#define " + name_1 + " " + v);
            }
            else if (typeof v === 'boolean') {
                if (v)
                    lines.push("#define " + name_1);
            }
            else {
                throw new Error('unknown define type');
            }
        }
    }
    return lines.join('\n') + '\n';
}
function getGlsl100FragPrefix(extensions, shaderExtensions) {
    var prefix = [];
    if (shaderExtensions.standardDerivatives) {
        prefix.push('#extension GL_OES_standard_derivatives : enable');
        prefix.push('#define enabledStandardDerivatives');
    }
    if (shaderExtensions.fragDepth) {
        if (extensions.fragDepth) {
            prefix.push('#extension GL_EXT_frag_depth : enable');
            prefix.push('#define enabledFragDepth');
        }
        else if (shaderExtensions.fragDepth === 'required') {
            throw new Error("required 'GL_EXT_frag_depth' extension not available");
        }
    }
    if (shaderExtensions.drawBuffers) {
        if (extensions.drawBuffers) {
            prefix.push('#extension GL_EXT_draw_buffers : require');
            prefix.push('#define requiredDrawBuffers');
            prefix.push('#define gl_FragColor gl_FragData[0]');
        }
        else if (shaderExtensions.drawBuffers === 'required') {
            throw new Error("required 'GL_EXT_draw_buffers' extension not available");
        }
    }
    if (shaderExtensions.shaderTextureLod) {
        if (extensions.shaderTextureLod) {
            prefix.push('#extension GL_EXT_shader_texture_lod : enable');
            prefix.push('#define enabledShaderTextureLod');
        }
        else if (shaderExtensions.shaderTextureLod === 'required') {
            throw new Error("required 'GL_EXT_shader_texture_lod' extension not available");
        }
    }
    if (extensions.depthTexture) {
        prefix.push('#define depthTextureSupport');
    }
    return prefix.join('\n') + '\n';
}
var glsl300VertPrefix = "#version 300 es\n#define attribute in\n#define varying out\n#define texture2D texture\n";
var glsl300FragPrefixCommon = "\nlayout(location = 0) out highp vec4 out_FragData0;\n\n#define varying in\n#define texture2D texture\n#define texture2DLodEXT textureLod\n\n#define gl_FragColor out_FragData0\n#define gl_FragDepthEXT gl_FragDepth\n\n#define depthTextureSupport\n";
function getGlsl300FragPrefix(gl, extensions, shaderExtensions) {
    var prefix = ['#version 300 es'];
    if (shaderExtensions.standardDerivatives) {
        prefix.push('#define enabledStandardDerivatives');
    }
    if (shaderExtensions.fragDepth) {
        prefix.push('#define enabledFragDepth');
    }
    if (shaderExtensions.drawBuffers) {
        prefix.push('#define requiredDrawBuffers');
        var maxDrawBuffers = gl.getParameter(gl.MAX_DRAW_BUFFERS);
        for (var i = 1, il = maxDrawBuffers; i < il; ++i) {
            prefix.push("layout(location = " + i + ") out highp vec4 out_FragData" + i + ";");
        }
    }
    if (shaderExtensions.shaderTextureLod) {
        prefix.push('#define enabledShaderTextureLod');
    }
    prefix.push(glsl300FragPrefixCommon);
    return prefix.join('\n') + '\n';
}
function transformGlsl300Frag(frag) {
    return frag.replace(/gl_FragData\[([0-9]+)\]/g, 'out_FragData$1');
}
function addShaderDefines(gl, extensions, defines, shaders) {
    var header = getDefinesCode(defines);
    var vertPrefix = compat_1.isWebGL2(gl) ? glsl300VertPrefix : '';
    var fragPrefix = compat_1.isWebGL2(gl)
        ? getGlsl300FragPrefix(gl, extensions, shaders.extensions)
        : getGlsl100FragPrefix(extensions, shaders.extensions);
    var frag = compat_1.isWebGL2(gl) ? transformGlsl300Frag(shaders.frag) : shaders.frag;
    return {
        id: shaderCodeId(),
        name: shaders.name,
        vert: "" + vertPrefix + header + shaders.vert,
        frag: "" + fragPrefix + header + frag,
        extensions: shaders.extensions
    };
}
exports.addShaderDefines = addShaderDefines;
//# sourceMappingURL=shader-code.js.map