"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.computeGaussianDensityTexture3d = exports.computeGaussianDensityTexture2d = exports.computeGaussianDensityTexture = exports.GaussianDensity = exports.computeGaussianDensity = exports.DefaultGaussianDensityProps = exports.DefaultGaussianDensityGPUProps = void 0;
var tslib_1 = require("tslib");
var mol_task_1 = require("../../mol-task");
var cpu_1 = require("./gaussian-density/cpu");
var gpu_1 = require("./gaussian-density/gpu");
// import { GaussianDensityGPU, GaussianDensityTexture } from './gaussian-density/gpu';
var GaussianDensityGPU = typeof document !== 'undefined'
    ? require('./gaussian-density/gpu').GaussianDensityGPU
    : void 0;
var GaussianDensityTexture = typeof document !== 'undefined'
    ? require('./gaussian-density/gpu').GaussianDensityTexture
    : void 0;
exports.DefaultGaussianDensityGPUProps = {
    resolution: 1,
    radiusOffset: 0,
    smoothness: 1.5,
};
exports.DefaultGaussianDensityProps = tslib_1.__assign(tslib_1.__assign({}, exports.DefaultGaussianDensityGPUProps), { useGpu: true });
function computeGaussianDensity(position, box, radius, props, webgl) {
    var _this = this;
    return mol_task_1.Task.create('Gaussian Density', function (ctx) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, GaussianDensity(ctx, position, box, radius, props, webgl)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    }); });
}
exports.computeGaussianDensity = computeGaussianDensity;
function GaussianDensity(ctx, position, box, radius, props, webgl) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!props.useGpu) return [3 /*break*/, 1];
                    if (!GaussianDensityGPU)
                        throw 'GPU computation not supported on this platform';
                    if (!webgl)
                        throw 'No WebGL context provided';
                    return [2 /*return*/, GaussianDensityGPU(position, box, radius, props, webgl)];
                case 1: return [4 /*yield*/, cpu_1.GaussianDensityCPU(ctx, position, box, radius, props)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.GaussianDensity = GaussianDensity;
function computeGaussianDensityTexture(position, box, radius, props, webgl, texture) {
    return _computeGaussianDensityTexture(webgl.isWebGL2 ? '3d' : '2d', position, box, radius, props, webgl, texture);
}
exports.computeGaussianDensityTexture = computeGaussianDensityTexture;
function computeGaussianDensityTexture2d(position, box, radius, props, webgl, texture) {
    return _computeGaussianDensityTexture('2d', position, box, radius, props, webgl, texture);
}
exports.computeGaussianDensityTexture2d = computeGaussianDensityTexture2d;
function computeGaussianDensityTexture3d(position, box, radius, props, webgl, texture) {
    return _computeGaussianDensityTexture('2d', position, box, radius, props, webgl, texture);
}
exports.computeGaussianDensityTexture3d = computeGaussianDensityTexture3d;
function _computeGaussianDensityTexture(type, position, box, radius, props, webgl, texture) {
    var _this = this;
    if (!GaussianDensityTexture)
        throw 'GPU computation not supported on this platform';
    return mol_task_1.Task.create('Gaussian Density', function (ctx) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2 /*return*/, type === '2d' ?
                    gpu_1.GaussianDensityTexture2d(webgl, position, box, radius, false, props, texture) :
                    gpu_1.GaussianDensityTexture3d(webgl, position, box, radius, props, texture)];
        });
    }); });
}
//# sourceMappingURL=gaussian-density.js.map