/**
 * Copyright (c) 2017-2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author Michael Krone <michael.krone@uni-tuebingen.de>
 */
import { PositionData } from '../common';
import { Box3D } from '../../geometry';
import { GaussianDensityGPUProps, GaussianDensityData, GaussianDensityTextureData } from '../gaussian-density';
import { Vec3, Tensor } from '../../linear-algebra';
import { WebGLContext } from '../../../mol-gl/webgl/context';
import { Texture } from '../../../mol-gl/webgl/texture';
import { ValueSpec, AttributeSpec, UniformSpec, TextureSpec, DefineSpec } from '../../../mol-gl/renderable/schema';
export declare const GaussianDensitySchema: {
    drawCount: ValueSpec<"number">;
    instanceCount: ValueSpec<"number">;
    aRadius: AttributeSpec<"float32">;
    aPosition: AttributeSpec<"float32">;
    aGroup: AttributeSpec<"float32">;
    uCurrentSlice: UniformSpec<"f">;
    uCurrentX: UniformSpec<"f">;
    uCurrentY: UniformSpec<"f">;
    uBboxMin: UniformSpec<"v3">;
    uBboxSize: UniformSpec<"v3">;
    uGridDim: UniformSpec<"v3">;
    uGridTexDim: UniformSpec<"v3">;
    uGridTexScale: UniformSpec<"v2">;
    uAlpha: UniformSpec<"f">;
    uResolution: UniformSpec<"f">;
    uRadiusFactorInv: UniformSpec<"f">;
    tMinDistanceTex: TextureSpec<"texture">;
    dGridTexType: DefineSpec<"string">;
    dCalcType: DefineSpec<"string">;
};
export declare function GaussianDensityGPU(position: PositionData, box: Box3D, radius: (index: number) => number, props: GaussianDensityGPUProps, webgl: WebGLContext): GaussianDensityData;
export declare function GaussianDensityTexture(webgl: WebGLContext, position: PositionData, box: Box3D, radius: (index: number) => number, props: GaussianDensityGPUProps, oldTexture?: Texture): GaussianDensityTextureData;
export declare function GaussianDensityTexture2d(webgl: WebGLContext, position: PositionData, box: Box3D, radius: (index: number) => number, powerOfTwo: boolean, props: GaussianDensityGPUProps, oldTexture?: Texture): GaussianDensityTextureData;
export declare function GaussianDensityTexture3d(webgl: WebGLContext, position: PositionData, box: Box3D, radius: (index: number) => number, props: GaussianDensityGPUProps, oldTexture?: Texture): GaussianDensityTextureData;
export declare function fieldFromTexture2d(ctx: WebGLContext, texture: Texture, dim: Vec3, texDim: Vec3): {
    field: Tensor;
    idField: Tensor;
};
