"use strict";
/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.trajectoryFrom3DG = exports.Format3dg = void 0;
var tslib_1 = require("tslib");
var mol_task_1 = require("../../mol-task");
var db_1 = require("../../mol-data/db");
var entity_1 = require("./common/entity");
var array_1 = require("../../mol-util/array");
var schema_1 = require("./basic/schema");
var parser_1 = require("./basic/parser");
function getBasic(table) {
    var entityIds = new Array(table._rowCount);
    var entityBuilder = new entity_1.EntityBuilder();
    var seqIdStarts = table.position.toArray({ array: Uint32Array });
    var seqIdEnds = new Uint32Array(table._rowCount);
    var stride = seqIdStarts[1] - seqIdStarts[0];
    var objectRadius = stride / 3500;
    for (var i = 0, il = table._rowCount; i < il; ++i) {
        var chr = table.chromosome.value(i);
        var entityId = entityBuilder.getEntityId(chr, 7 /* DNA */, chr);
        entityIds[i] = entityId;
        seqIdEnds[i] = seqIdStarts[i] + stride - 1;
    }
    var ihm_sphere_obj_site = db_1.Table.ofPartialColumns(schema_1.BasicSchema.ihm_sphere_obj_site, {
        id: db_1.Column.ofIntArray(array_1.fillSerial(new Uint32Array(table._rowCount))),
        entity_id: db_1.Column.ofStringArray(entityIds),
        seq_id_begin: db_1.Column.ofIntArray(seqIdStarts),
        seq_id_end: db_1.Column.ofIntArray(seqIdEnds),
        asym_id: table.chromosome,
        Cartn_x: db_1.Column.ofFloatArray(db_1.Column.mapToArray(table.x, function (x) { return x * 10; }, Float32Array)),
        Cartn_y: db_1.Column.ofFloatArray(db_1.Column.mapToArray(table.y, function (y) { return y * 10; }, Float32Array)),
        Cartn_z: db_1.Column.ofFloatArray(db_1.Column.mapToArray(table.z, function (z) { return z * 10; }, Float32Array)),
        object_radius: db_1.Column.ofConst(objectRadius, table._rowCount, db_1.Column.Schema.float),
        rmsf: db_1.Column.ofConst(0, table._rowCount, db_1.Column.Schema.float),
        model_id: db_1.Column.ofConst(1, table._rowCount, db_1.Column.Schema.int),
    }, table._rowCount);
    return schema_1.createBasic({
        entity: entityBuilder.getEntityTable(),
        ihm_model_list: db_1.Table.ofPartialColumns(schema_1.BasicSchema.ihm_model_list, {
            model_id: db_1.Column.ofIntArray([1]),
            model_name: db_1.Column.ofStringArray(['3DG Model']),
        }, 1),
        ihm_sphere_obj_site: ihm_sphere_obj_site
    });
}
var Format3dg;
(function (Format3dg) {
    function is(x) {
        return x.kind === '3dg';
    }
    Format3dg.is = is;
    function from3dg(file3dg) {
        return { kind: '3dg', name: '3DG', data: file3dg };
    }
    Format3dg.from3dg = from3dg;
})(Format3dg || (Format3dg = {}));
exports.Format3dg = Format3dg;
function trajectoryFrom3DG(file3dg) {
    var _this = this;
    return mol_task_1.Task.create('Parse 3DG', function (ctx) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var format, basic;
        return tslib_1.__generator(this, function (_a) {
            format = Format3dg.from3dg(file3dg);
            basic = getBasic(file3dg.table);
            return [2 /*return*/, parser_1.createModels(basic, format, ctx)];
        });
    }); });
}
exports.trajectoryFrom3DG = trajectoryFrom3DG;
//# sourceMappingURL=3dg.js.map