"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.explodeStructure = exports.unwindStructureAssembly = void 0;
var geometry_1 = require("../../mol-math/geometry");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var _unwindMatrix = linear_algebra_1.Mat4.zero();
function unwindStructureAssembly(structure, unitTransforms, t) {
    for (var i = 0, _i = structure.units.length; i < _i; i++) {
        var u = structure.units[i];
        geometry_1.SymmetryOperator.lerpFromIdentity(_unwindMatrix, u.conformation.operator, t);
        unitTransforms.setTransform(_unwindMatrix, u);
    }
}
exports.unwindStructureAssembly = unwindStructureAssembly;
var _centerVec = linear_algebra_1.Vec3.zero(), _transVec = linear_algebra_1.Vec3.zero(), _transMat = linear_algebra_1.Mat4.zero();
function explodeStructure(structure, unitTransforms, t) {
    var boundary = structure.boundary.sphere;
    var d = boundary.radius * t;
    for (var i = 0, _i = structure.units.length; i < _i; i++) {
        var u = structure.units[i];
        linear_algebra_1.Vec3.transformMat4(_centerVec, u.lookup3d.boundary.sphere.center, u.conformation.operator.matrix);
        linear_algebra_1.Vec3.sub(_transVec, _centerVec, boundary.center);
        linear_algebra_1.Vec3.setMagnitude(_transVec, _transVec, d);
        linear_algebra_1.Mat4.fromTranslation(_transMat, _transVec);
        unitTransforms.setTransform(_transMat, u);
    }
}
exports.explodeStructure = explodeStructure;
//# sourceMappingURL=helpers.js.map