"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableLegend = exports.ScaleLegend = exports.legendFor = void 0;
var tslib_1 = require("tslib");
var color_1 = require("../../mol-util/color");
var React = tslib_1.__importStar(require("react"));
function legendFor(legend) {
    switch (legend.kind) {
        case 'scale-legend': return ScaleLegend;
        case 'table-legend': return TableLegend;
        default:
            var _ = legend;
            console.warn(_ + " has no associated UI component");
            return void 0;
    }
}
exports.legendFor = legendFor;
var ScaleLegend = /** @class */ (function (_super) {
    tslib_1.__extends(ScaleLegend, _super);
    function ScaleLegend() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ScaleLegend.prototype.render = function () {
        var legend = this.props.legend;
        var colors = legend.colors.map(function (c) { return Array.isArray(c) ? color_1.Color.toStyle(c[0]) + " " + 100 * c[1] + "%" : color_1.Color.toStyle(c); }).join(', ');
        return React.createElement("div", { className: 'msp-scale-legend' },
            React.createElement("div", { style: { background: "linear-gradient(to right, " + colors + ")" } },
                React.createElement("span", { style: { float: 'left' } }, legend.minLabel),
                React.createElement("span", { style: { float: 'right' } }, legend.maxLabel)));
    };
    return ScaleLegend;
}(React.PureComponent));
exports.ScaleLegend = ScaleLegend;
var TableLegend = /** @class */ (function (_super) {
    tslib_1.__extends(TableLegend, _super);
    function TableLegend() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TableLegend.prototype.render = function () {
        var legend = this.props.legend;
        return React.createElement("div", { className: 'msp-table-legend' }, legend.table.map(function (value, i) {
            var name = value[0], color = value[1];
            return React.createElement("div", { key: i },
                React.createElement("div", { className: 'msp-table-legend-color', style: { backgroundColor: color_1.Color.toStyle(color) } }),
                React.createElement("div", { className: 'msp-table-legend-text' }, name));
        }));
    };
    return TableLegend;
}(React.PureComponent));
exports.TableLegend = TableLegend;
//# sourceMappingURL=legend.js.map