"use strict";
/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewportCanvas = void 0;
var tslib_1 = require("tslib");
var React = tslib_1.__importStar(require("react"));
var base_1 = require("../base");
var util_1 = require("../../mol-canvas3d/util");
var rxjs_1 = require("rxjs");
var debounceTime_1 = require("rxjs/internal/operators/debounceTime");
var config_1 = require("../../mol-plugin/config");
var color_1 = require("../../mol-util/color");
var ViewportCanvas = /** @class */ (function (_super) {
    tslib_1.__extends(ViewportCanvas, _super);
    function ViewportCanvas() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.container = React.createRef();
        _this.canvas = React.createRef();
        _this.state = {
            noWebGl: false,
            showLogo: true
        };
        _this.handleLogo = function () {
            var _a;
            _this.setState({ showLogo: !((_a = _this.plugin.canvas3d) === null || _a === void 0 ? void 0 : _a.reprCount.value) });
        };
        _this.handleResize = function () {
            var container = _this.container.current;
            var canvas = _this.canvas.current;
            if (container && canvas) {
                var pixelScale = _this.plugin.config.get(config_1.PluginConfig.General.PixelScale) || 1;
                util_1.resizeCanvas(canvas, container, pixelScale);
                var _a = color_1.Color.toRgbNormalized(_this.plugin.canvas3d.props.renderer.backgroundColor), r = _a[0], g = _a[1], b = _a[2];
                var a = _this.plugin.canvas3d.props.transparentBackground ? 0 : 1;
                _this.plugin.canvas3d.webgl.clear(r, g, b, a);
                _this.plugin.canvas3d.handleResize();
            }
        };
        return _this;
    }
    ViewportCanvas.prototype.componentDidMount = function () {
        var _this = this;
        if (!this.canvas.current || !this.container.current || !this.plugin.initViewer(this.canvas.current, this.container.current)) {
            this.setState({ noWebGl: true });
            return;
        }
        this.handleLogo();
        this.handleResize();
        var canvas3d = this.plugin.canvas3d;
        this.subscribe(canvas3d.reprCount, this.handleLogo);
        var resized = new rxjs_1.Subject();
        var resize = function () { return resized.next(); };
        this.subscribe(resized.pipe(debounceTime_1.debounceTime(1000 / 24)), function () { return _this.handleResize(); });
        this.subscribe(canvas3d.input.resize, resize);
        this.subscribe(canvas3d.interaction.click, function (e) { return _this.plugin.behaviors.interaction.click.next(e); });
        this.subscribe(canvas3d.interaction.drag, function (e) { return _this.plugin.behaviors.interaction.drag.next(e); });
        this.subscribe(canvas3d.interaction.hover, function (e) { return _this.plugin.behaviors.interaction.hover.next(e); });
        this.subscribe(this.plugin.layout.events.updated, resize);
    };
    ViewportCanvas.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        // TODO viewer cleanup
    };
    ViewportCanvas.prototype.renderMissing = function () {
        if (this.props.noWebGl) {
            var C = this.props.noWebGl;
            return React.createElement(C, null);
        }
        return React.createElement("div", { className: 'msp-no-webgl' },
            React.createElement("div", null,
                React.createElement("p", null,
                    React.createElement("b", null, "WebGL does not seem to be available.")),
                React.createElement("p", null, "This can be caused by an outdated browser, graphics card driver issue, or bad weather. Sometimes, just restarting the browser helps."),
                React.createElement("p", null,
                    "For a list of supported browsers, refer to ",
                    React.createElement("a", { href: 'http://caniuse.com/#feat=webgl', target: '_blank' }, "http://caniuse.com/#feat=webgl"),
                    ".")));
    };
    ViewportCanvas.prototype.render = function () {
        if (this.state.noWebGl)
            return this.renderMissing();
        var Logo = this.props.logo;
        return React.createElement("div", { className: this.props.parentClassName || 'msp-viewport', style: this.props.parentStyle },
            React.createElement("div", { className: this.props.hostClassName || 'msp-viewport-host3d', style: this.props.hostStyle, ref: this.container },
                React.createElement("canvas", { ref: this.canvas })),
            (this.state.showLogo && Logo) && React.createElement(Logo, null));
    };
    return ViewportCanvas;
}(base_1.PluginUIComponent));
exports.ViewportCanvas = ViewportCanvas;
//# sourceMappingURL=canvas.js.map