"use strict";
/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.HelpContent = exports.ViewportHelpContent = exports.HelpGroup = exports.HelpText = exports.BindingsHelp = void 0;
var tslib_1 = require("tslib");
var React = tslib_1.__importStar(require("react"));
var binding_1 = require("../../mol-util/binding");
var base_1 = require("../base");
var mol_state_1 = require("../../mol-state");
var representation_1 = require("../../mol-plugin/behavior/dynamic/representation");
var representation_2 = require("../../mol-plugin/behavior/dynamic/representation");
var icons_1 = require("../controls/icons");
var common_1 = require("../controls/common");
function getBindingsList(bindings) {
    return Object.keys(bindings).map(function (k) { return [k, bindings[k]]; });
}
var BindingsHelp = /** @class */ (function (_super) {
    tslib_1.__extends(BindingsHelp, _super);
    function BindingsHelp() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BindingsHelp.prototype.getBindingComponents = function () {
        var bindingsList = getBindingsList(this.props.bindings);
        return React.createElement(React.Fragment, null, bindingsList.map(function (value) {
            var name = value[0], binding = value[1];
            return !binding_1.Binding.isEmpty(binding)
                ? React.createElement("div", { key: name, style: { marginBottom: '6px' } },
                    React.createElement("b", null, binding.action),
                    React.createElement("br", null),
                    React.createElement("span", { dangerouslySetInnerHTML: { __html: binding_1.Binding.format(binding, name) } }))
                : null;
        }));
    };
    BindingsHelp.prototype.render = function () {
        return React.createElement(HelpText, null, this.getBindingComponents());
    };
    return BindingsHelp;
}(React.PureComponent));
exports.BindingsHelp = BindingsHelp;
var HelpText = /** @class */ (function (_super) {
    tslib_1.__extends(HelpText, _super);
    function HelpText() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    HelpText.prototype.render = function () {
        return React.createElement("div", { className: 'msp-help-text' },
            React.createElement("div", null, this.props.children));
    };
    return HelpText;
}(React.PureComponent));
exports.HelpText = HelpText;
var HelpGroup = /** @class */ (function (_super) {
    tslib_1.__extends(HelpGroup, _super);
    function HelpGroup() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            header: _this.props.header,
            isExpanded: !!_this.props.initiallyExpanded
        };
        _this.toggleExpanded = function () { return _this.setState({ isExpanded: !_this.state.isExpanded }); };
        return _this;
    }
    HelpGroup.prototype.render = function () {
        return React.createElement("div", { className: 'msp-control-group-wrapper' },
            React.createElement("div", { className: 'msp-control-group-header' },
                React.createElement(common_1.Button, { onClick: this.toggleExpanded },
                    React.createElement(icons_1.Icon, { svg: this.state.isExpanded ? icons_1.ArrowDropDownSvg : icons_1.ArrowRightSvg }),
                    this.props.header)),
            this.state.isExpanded && React.createElement("div", { className: 'msp-control-offset', style: { display: this.state.isExpanded ? 'block' : 'none' } }, this.props.children));
    };
    return HelpGroup;
}(React.PureComponent));
exports.HelpGroup = HelpGroup;
function HelpSection(props) {
    return React.createElement("div", { className: 'msp-simple-help-section' }, props.header);
}
var ViewportHelpContent = /** @class */ (function (_super) {
    tslib_1.__extends(ViewportHelpContent, _super);
    function ViewportHelpContent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ViewportHelpContent.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.events.canvas3d.settingsUpdated, function () { return _this.forceUpdate(); });
    };
    ViewportHelpContent.prototype.render = function () {
        var interactionBindings = {};
        this.plugin.spec.behaviors.forEach(function (b) {
            var bindings = b.defaultParams.bindings;
            if (bindings)
                Object.assign(interactionBindings, bindings);
        });
        return React.createElement(React.Fragment, null,
            (!this.props.selectOnly && this.plugin.canvas3d) && React.createElement(HelpGroup, { key: 'trackball', header: 'Moving in 3D' },
                React.createElement(BindingsHelp, { bindings: this.plugin.canvas3d.props.trackball.bindings })),
            React.createElement(HelpGroup, { key: 'interactions', header: 'Mouse Controls' },
                React.createElement(BindingsHelp, { bindings: interactionBindings })));
    };
    return ViewportHelpContent;
}(base_1.PluginUIComponent));
exports.ViewportHelpContent = ViewportHelpContent;
var HelpContent = /** @class */ (function (_super) {
    tslib_1.__extends(HelpContent, _super);
    function HelpContent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    HelpContent.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.events.canvas3d.settingsUpdated, function () { return _this.forceUpdate(); });
    };
    HelpContent.prototype.formatTriggers = function (binding) {
        return binding.triggers.map(function (t) { return binding_1.Binding.Trigger.format(t); }).join(' or ');
    };
    HelpContent.prototype.getTriggerFor = function (transformer, name) {
        var state = this.plugin.state.behaviors;
        var selections = state.select(mol_state_1.StateSelection.Generators.ofTransformer(transformer));
        var params = selections.length === 1 ? selections[0].params : undefined;
        var bindings = params ? params.values.bindings : {};
        var binding = name in bindings ? bindings[name] : binding_1.Binding.Empty;
        return this.formatTriggers(binding);
    };
    HelpContent.prototype.render = function () {
        var selectToggleTriggers = this.getTriggerFor(representation_1.SelectLoci, 'clickSelectToggle');
        var focusTriggers = this.getTriggerFor(representation_2.FocusLoci, 'clickFocus');
        // TODO: interactive help, for example for density
        return React.createElement("div", null,
            React.createElement(HelpSection, { header: 'Interface Controls' }),
            React.createElement(HelpGroup, { header: 'Inline Help' },
                React.createElement(HelpText, null, "Many user interface elements show a little questionmark icon when hovered over. Clicking the icon toggles the display of an inline help text."),
                React.createElement(HelpText, null, "Tooltips may provide additional information on a user interface element and are shown when hovering over it with the mouse.")),
            React.createElement(HelpGroup, { header: 'Selections' },
                React.createElement(HelpText, null,
                    "The viewer allows changing colors and representations for selections of atoms, residues or chains. Selections can be created by",
                    React.createElement("ul", { style: { paddingLeft: '20px' } },
                        React.createElement("li", null,
                            "picking elements on the 3D canvas or the sequence view using the mouse, e.g. toggle selection using ",
                            selectToggleTriggers,
                            " (for more see help section on ",
                            React.createElement("i", null, "Mouse Controls"),
                            ")"),
                        React.createElement("li", null,
                            "using the ",
                            React.createElement("i", null, "Add"),
                            ", ",
                            React.createElement("i", null, "Remove"),
                            " and ",
                            React.createElement("i", null, "Only"),
                            " dropdown buttons in the ",
                            React.createElement("i", null, "Manage Selection"),
                            " panel which allow modifing the current selection by predefined sets")))),
            React.createElement(HelpGroup, { header: 'Coloring' },
                React.createElement(HelpText, null,
                    "There are two ways to color structures. Every representation (e.g. cartoon or spacefill) has a color theme which can be changed using the dropdown for each representation in the ",
                    React.createElement("i", null, "Structure Settings"),
                    " panel. Additionally any selection atoms, residues or chains can by given a custom color. For that, first select the parts of the structure to be colored (see help section on ",
                    React.createElement("i", null, "Selections"),
                    ") and, second, choose a color from the color dropdown botton in the ",
                    React.createElement("i", null, "Selection"),
                    " row of the ",
                    React.createElement("i", null, "Change Representation"),
                    " panel. The theme color can be seen as a base color that is overpainted by the custom color. Custom colors can be removed for a selection with the 'Clear' option in the color dropdown.")),
            React.createElement(HelpGroup, { header: 'Representations' },
                React.createElement(HelpText, null,
                    "Structures can be shown with many different representations (e.g. cartoon or spacefill). The ",
                    React.createElement("i", null, "Change Representation"),
                    " panel offers a collection of predefined styles which can be applied using the ",
                    React.createElement("i", null, "Preset"),
                    " dropdown button. Additionally any selection atoms, residues or chains can by shown with a custom representation. For that, first select the parts of the structure to be mofified (see help section on ",
                    React.createElement("i", null, "Selections"),
                    ") and, second, choose a representation to hide or show from the ",
                    React.createElement("i", null, "Show"),
                    " and ",
                    React.createElement("i", null, "Hide"),
                    " dropdown bottons in the ",
                    React.createElement("i", null, "Selection"),
                    " row of the ",
                    React.createElement("i", null, "Change Representation"),
                    " panel. The ",
                    React.createElement("i", null, "Everything"),
                    " row applies the action to the whole structure instead of the current selection.")),
            React.createElement(HelpGroup, { header: 'Surroundings' },
                React.createElement(HelpText, null,
                    "To show the surroundings of a residue or ligand, click it in the 3D scene or in the sequence widget using ",
                    focusTriggers,
                    ".")),
            React.createElement(HelpSection, { header: 'How-to Guides' }),
            React.createElement(HelpGroup, { header: 'Create an Image' },
                React.createElement(HelpText, null,
                    React.createElement("p", null,
                        "Use the ",
                        React.createElement(icons_1.Icon, { svg: icons_1.CameraSvg }),
                        " icon in the viewport to bring up the screenshot controls."),
                    React.createElement("p", null,
                        "To adjust the size of the image, use the ",
                        React.createElement("i", null, "Resolution"),
                        " dropdown."))),
            React.createElement(HelpSection, { header: 'Mouse Controls' }),
            React.createElement(ViewportHelpContent, null));
    };
    return HelpContent;
}(base_1.PluginUIComponent));
exports.HelpContent = HelpContent;
//# sourceMappingURL=help.js.map