"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPluginAsync = exports.createPlugin = exports.DefaultPluginSpec = void 0;
var tslib_1 = require("tslib");
var React = tslib_1.__importStar(require("react"));
var ReactDOM = tslib_1.__importStar(require("react-dom"));
var actions_1 = require("../mol-plugin-state/actions");
var transforms_1 = require("../mol-plugin-state/transforms");
var volume_1 = require("../mol-plugin-ui/custom/volume");
var plugin_1 = require("../mol-plugin-ui/plugin");
var behavior_1 = require("./behavior");
var structure_focus_representation_1 = require("./behavior/dynamic/selection/structure-focus-representation");
var transformers_1 = require("./behavior/dynamic/volume-streaming/transformers");
var context_1 = require("./context");
var spec_1 = require("./spec");
var volume_2 = require("../mol-plugin-state/actions/volume");
var model_index_1 = require("../mol-plugin-state/animation/built-in/model-index");
var assembly_unwind_1 = require("../mol-plugin-state/animation/built-in/assembly-unwind");
var camera_spin_1 = require("../mol-plugin-state/animation/built-in/camera-spin");
var state_snapshots_1 = require("../mol-plugin-state/animation/built-in/state-snapshots");
exports.DefaultPluginSpec = {
    actions: [
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.DownloadStructure),
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.AddTrajectory),
        spec_1.PluginSpec.Action(actions_1.StateActions.Volume.DownloadDensity),
        spec_1.PluginSpec.Action(actions_1.StateActions.DataFormat.DownloadFile),
        spec_1.PluginSpec.Action(actions_1.StateActions.DataFormat.OpenFiles),
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.EnableModelCustomProps),
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.EnableStructureCustomProps),
        // Volume streaming
        spec_1.PluginSpec.Action(transformers_1.InitVolumeStreaming),
        spec_1.PluginSpec.Action(transformers_1.BoxifyVolumeStreaming),
        spec_1.PluginSpec.Action(transformers_1.CreateVolumeStreamingBehavior),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.Download),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.ParseCif),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.ParseCcp4),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.ParseDsn6),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.TrajectoryFromMmCif),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.TrajectoryFromCifCore),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.TrajectoryFromPDB),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.TransformStructureConformation),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.StructureFromModel),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.StructureFromTrajectory),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.ModelFromTrajectory),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.StructureSelectionFromScript),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureSelectionsDistance3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureSelectionsAngle3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureSelectionsDihedral3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureSelectionsLabel3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureSelectionsOrientation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.ModelUnitcell3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.ExplodeStructureRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.UnwindStructureAssemblyRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.OverpaintStructureRepresentation3DFromScript),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.TransparencyStructureRepresentation3DFromScript),
        spec_1.PluginSpec.Action(volume_2.AssignColorVolume),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Volume.VolumeFromCcp4),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Volume.VolumeFromDsn6),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Volume.VolumeFromCube),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Volume.VolumeFromDx),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.VolumeRepresentation3D),
    ],
    behaviors: [
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Representation.HighlightLoci),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Representation.SelectLoci),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Representation.DefaultLociLabelProvider),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Representation.FocusLoci),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Camera.FocusLoci),
        spec_1.PluginSpec.Behavior(structure_focus_representation_1.StructureFocusRepresentation),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.StructureInfo),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.AccessibleSurfaceArea),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.Interactions),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.SecondaryStructure),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.ValenceModel),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.CrossLinkRestraint),
    ],
    customParamEditors: [
        [transformers_1.CreateVolumeStreamingBehavior, volume_1.VolumeStreamingCustomControls]
    ],
    animations: [
        model_index_1.AnimateModelIndex,
        camera_spin_1.AnimateCameraSpin,
        state_snapshots_1.AnimateStateSnapshots,
        assembly_unwind_1.AnimateAssemblyUnwind
    ]
};
function createPlugin(target, spec) {
    var ctx = new context_1.PluginContext(spec || exports.DefaultPluginSpec);
    ctx.init();
    ReactDOM.render(React.createElement(plugin_1.Plugin, { plugin: ctx }), target);
    return ctx;
}
exports.createPlugin = createPlugin;
/** Returns the instance of the plugin after all behaviors have been initialized */
function createPluginAsync(target, spec) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var ctx, init;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    ctx = new context_1.PluginContext(spec || exports.DefaultPluginSpec);
                    init = ctx.init();
                    ReactDOM.render(React.createElement(plugin_1.Plugin, { plugin: ctx }), target);
                    return [4 /*yield*/, init];
                case 1:
                    _a.sent();
                    return [2 /*return*/, ctx];
            }
        });
    });
}
exports.createPluginAsync = createPluginAsync;
//# sourceMappingURL=index.js.map