/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
/// <reference types="react" />
import { StateTransformer, StateAction } from '../mol-state';
import { StateTransformParameters } from '../mol-plugin-ui/state/common';
import { PluginLayoutStateProps } from './layout';
import { PluginStateAnimation } from '../mol-plugin-state/animation/model';
import { PluginConfigItem } from './config';
import { PartialCanvas3DProps } from '../mol-canvas3d/canvas3d';
import { DataFormatProvider } from '../mol-plugin-state/formats/provider';
export { PluginSpec };
interface PluginSpec {
    actions: PluginSpec.Action[];
    behaviors: PluginSpec.Behavior[];
    animations?: PluginStateAnimation[];
    customParamEditors?: [StateAction | StateTransformer, StateTransformParameters.Class][];
    customFormats?: [string, DataFormatProvider][];
    layout?: {
        initial?: Partial<PluginLayoutStateProps>;
        controls?: PluginSpec.LayoutControls;
    };
    components?: {
        remoteState?: 'none' | 'default';
        structureTools?: React.ComponentClass;
        viewport?: {
            view?: React.ComponentClass;
            controls?: React.ComponentClass;
            canvas3d?: PartialCanvas3DProps;
        };
        hideTaskOverlay?: boolean;
    };
    config?: [PluginConfigItem, unknown][];
}
declare namespace PluginSpec {
    interface Action {
        action: StateAction | StateTransformer;
        customControl?: StateTransformParameters.Class;
        autoUpdate?: boolean;
    }
    function Action(action: StateAction | StateTransformer, params?: {
        customControl?: StateTransformParameters.Class;
        autoUpdate?: boolean;
    }): Action;
    interface Behavior {
        transformer: StateTransformer;
        defaultParams?: any;
    }
    function Behavior<T extends StateTransformer>(transformer: T, defaultParams?: Partial<StateTransformer.Params<T>>): Behavior;
    interface LayoutControls {
        top?: React.ComponentClass | 'none';
        left?: React.ComponentClass | 'none';
        right?: React.ComponentClass | 'none';
        bottom?: React.ComponentClass | 'none';
    }
}
