"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.BallAndStickRepresentationProvider = exports.BallAndStickRepresentation = exports.getBallAndStickParams = exports.BallAndStickParams = void 0;
var tslib_1 = require("tslib");
var element_sphere_1 = require("../visual/element-sphere");
var bond_intra_unit_cylinder_1 = require("../visual/bond-intra-unit-cylinder");
var bond_inter_unit_cylinder_1 = require("../visual/bond-inter-unit-cylinder");
var param_definition_1 = require("../../../mol-util/param-definition");
var units_representation_1 = require("../units-representation");
var complex_representation_1 = require("../complex-representation");
var representation_1 = require("../representation");
var representation_2 = require("../../../mol-repr/representation");
var params_1 = require("../params");
var BallAndStickVisuals = {
    'element-sphere': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Element sphere mesh', ctx, getParams, element_sphere_1.getElementSphereVisual(ctx.webgl)); },
    'intra-bond': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Intra-unit bond cylinder', ctx, getParams, bond_intra_unit_cylinder_1.IntraUnitBondCylinderVisual); },
    'inter-bond': function (ctx, getParams) { return complex_representation_1.ComplexRepresentation('Inter-unit bond cylinder', ctx, getParams, bond_inter_unit_cylinder_1.InterUnitBondCylinderVisual); },
};
exports.BallAndStickParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, element_sphere_1.ElementSphereParams), bond_intra_unit_cylinder_1.IntraUnitBondCylinderParams), bond_inter_unit_cylinder_1.InterUnitBondCylinderParams), { unitKinds: params_1.getUnitKindsParam(['atomic']), sizeFactor: param_definition_1.ParamDefinition.Numeric(0.15, { min: 0.01, max: 10, step: 0.01 }), sizeAspectRatio: param_definition_1.ParamDefinition.Numeric(2 / 3, { min: 0.01, max: 3, step: 0.01 }), visuals: param_definition_1.ParamDefinition.MultiSelect(['element-sphere', 'intra-bond', 'inter-bond'], param_definition_1.ParamDefinition.objectToOptions(BallAndStickVisuals)) });
function getBallAndStickParams(ctx, structure) {
    return param_definition_1.ParamDefinition.clone(exports.BallAndStickParams);
}
exports.getBallAndStickParams = getBallAndStickParams;
function BallAndStickRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Ball & Stick', ctx, getParams, representation_1.StructureRepresentationStateBuilder, BallAndStickVisuals);
}
exports.BallAndStickRepresentation = BallAndStickRepresentation;
exports.BallAndStickRepresentationProvider = representation_1.StructureRepresentationProvider({
    name: 'ball-and-stick',
    label: 'Ball & Stick',
    description: 'Displays atoms as spheres and bonds as cylinders.',
    factory: BallAndStickRepresentation,
    getParams: getBallAndStickParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.BallAndStickParams),
    defaultColorTheme: { name: 'element-symbol' },
    defaultSizeTheme: { name: 'physical' },
    isApplicable: function (structure) { return structure.elementCount > 0; }
});
//# sourceMappingURL=ball-and-stick.js.map