/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { StructureRepresentation, StructureRepresentationProvider } from '../representation';
import { RepresentationParamsGetter, RepresentationContext } from '../../../mol-repr/representation';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
import { Structure } from '../../../mol-model/structure';
export declare const LineParams: {
    sizeFactor: PD.Numeric;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    visuals: PD.MultiSelect<"intra-bond" | "inter-bond", string[]>;
    includeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    excludeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
};
export declare type LineParams = typeof LineParams;
export declare function getLineParams(ctx: ThemeRegistryContext, structure: Structure): {
    sizeFactor: PD.Numeric;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    visuals: PD.MultiSelect<"intra-bond" | "inter-bond", string[]>;
    includeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    excludeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
};
export declare type LineRepresentation = StructureRepresentation<LineParams>;
export declare function LineRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<Structure, LineParams>): LineRepresentation;
export declare const LineRepresentationProvider: StructureRepresentationProvider<{
    sizeFactor: PD.Numeric;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    visuals: PD.MultiSelect<"intra-bond" | "inter-bond", string[]>;
    includeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    excludeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
}, "line">;
