"use strict";
/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.LineRepresentationProvider = exports.LineRepresentation = exports.getLineParams = exports.LineParams = void 0;
var tslib_1 = require("tslib");
var bond_intra_unit_line_1 = require("../visual/bond-intra-unit-line");
var bond_inter_unit_line_1 = require("../visual/bond-inter-unit-line");
var param_definition_1 = require("../../../mol-util/param-definition");
var units_representation_1 = require("../units-representation");
var complex_representation_1 = require("../complex-representation");
var representation_1 = require("../representation");
var representation_2 = require("../../../mol-repr/representation");
var params_1 = require("../params");
var LineVisuals = {
    'intra-bond': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Intra-unit bond line', ctx, getParams, bond_intra_unit_line_1.IntraUnitBondLineVisual); },
    'inter-bond': function (ctx, getParams) { return complex_representation_1.ComplexRepresentation('Inter-unit bond line', ctx, getParams, bond_inter_unit_line_1.InterUnitBondLineVisual); },
};
exports.LineParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, bond_intra_unit_line_1.IntraUnitBondLineParams), bond_inter_unit_line_1.InterUnitBondLineParams), { sizeFactor: param_definition_1.ParamDefinition.Numeric(1.0, { min: 0.01, max: 10, step: 0.01 }), unitKinds: params_1.getUnitKindsParam(['atomic']), visuals: param_definition_1.ParamDefinition.MultiSelect(['intra-bond', 'inter-bond'], param_definition_1.ParamDefinition.objectToOptions(LineVisuals)) });
function getLineParams(ctx, structure) {
    return param_definition_1.ParamDefinition.clone(exports.LineParams);
}
exports.getLineParams = getLineParams;
function LineRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Line', ctx, getParams, representation_1.StructureRepresentationStateBuilder, LineVisuals);
}
exports.LineRepresentation = LineRepresentation;
exports.LineRepresentationProvider = representation_1.StructureRepresentationProvider({
    name: 'line',
    label: 'Line',
    description: 'Displays bonds as lines.',
    factory: LineRepresentation,
    getParams: getLineParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.LineParams),
    defaultColorTheme: { name: 'element-symbol' },
    defaultSizeTheme: { name: 'uniform' },
    isApplicable: function (structure) { return structure.elementCount > 0; }
});
//# sourceMappingURL=line.js.map