"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnitsGaussianDensityVolumeVisual = exports.UnitsGaussianDensityVolumeParams = exports.GaussianDensityVolumeVisual = exports.GaussianDensityVolumeParams = void 0;
var tslib_1 = require("tslib");
var param_definition_1 = require("../../../mol-util/param-definition");
var gaussian_1 = require("./util/gaussian");
var direct_volume_1 = require("../../../mol-geo/geometry/direct-volume/direct-volume");
var complex_visual_1 = require("../complex-visual");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var element_1 = require("./util/element");
var geometry_1 = require("../../../mol-math/geometry");
var units_visual_1 = require("../units-visual");
var common_1 = require("./util/common");
function createGaussianDensityVolume(ctx, structure, theme, props, directVolume) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var runtime, webgl, p, oldTexture, densityTextureData, transform, texture, bbox, gridDim, stats, unitToCartn, cellDim, vol, sphere;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    runtime = ctx.runtime, webgl = ctx.webgl;
                    if (webgl === undefined)
                        throw new Error('createGaussianDensityVolume requires `webgl` object in VisualContext');
                    p = tslib_1.__assign(tslib_1.__assign({}, props), { useGpu: true });
                    oldTexture = directVolume ? directVolume.gridTexture.ref.value : undefined;
                    return [4 /*yield*/, gaussian_1.computeStructureGaussianDensityTexture(structure, p, webgl, oldTexture).runInContext(runtime)];
                case 1:
                    densityTextureData = _a.sent();
                    transform = densityTextureData.transform, texture = densityTextureData.texture, bbox = densityTextureData.bbox, gridDim = densityTextureData.gridDim;
                    stats = { min: 0, max: 1, mean: 0.04, sigma: 0.01 };
                    unitToCartn = linear_algebra_1.Mat4.mul(linear_algebra_1.Mat4(), transform, linear_algebra_1.Mat4.fromScaling(linear_algebra_1.Mat4(), gridDim));
                    cellDim = linear_algebra_1.Mat4.getScaling(linear_algebra_1.Vec3(), transform);
                    vol = direct_volume_1.DirectVolume.create(bbox, gridDim, transform, unitToCartn, cellDim, texture, stats, true, directVolume);
                    sphere = geometry_1.Sphere3D.expand(geometry_1.Sphere3D(), structure.boundary.sphere, props.radiusOffset + common_1.getStructureExtraRadius(structure));
                    vol.setBoundingSphere(sphere);
                    return [2 /*return*/, vol];
            }
        });
    });
}
exports.GaussianDensityVolumeParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, complex_visual_1.ComplexDirectVolumeParams), gaussian_1.GaussianDensityTextureParams), { ignoreHydrogens: param_definition_1.ParamDefinition.Boolean(false) });
function GaussianDensityVolumeVisual(materialId) {
    return complex_visual_1.ComplexDirectVolumeVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.GaussianDensityVolumeParams),
        createGeometry: createGaussianDensityVolume,
        createLocationIterator: element_1.ElementIterator.fromStructure,
        getLoci: element_1.getSerialElementLoci,
        eachLocation: element_1.eachSerialElement,
        setUpdateState: function (state, newProps, currentProps) {
            if (newProps.resolution !== currentProps.resolution)
                state.createGeometry = true;
            if (newProps.radiusOffset !== currentProps.radiusOffset)
                state.createGeometry = true;
            if (newProps.smoothness !== currentProps.smoothness)
                state.createGeometry = true;
            if (newProps.ignoreHydrogens !== currentProps.ignoreHydrogens)
                state.createGeometry = true;
            if (newProps.traceOnly !== currentProps.traceOnly)
                state.createGeometry = true;
            if (newProps.includeParent !== currentProps.includeParent)
                state.createGeometry = true;
        }
    }, materialId);
}
exports.GaussianDensityVolumeVisual = GaussianDensityVolumeVisual;
//
function createUnitsGaussianDensityVolume(ctx, unit, structure, theme, props, directVolume) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var runtime, webgl, p, oldTexture, densityTextureData, transform, texture, bbox, gridDim, stats, unitToCartn, cellDim, vol, sphere;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    runtime = ctx.runtime, webgl = ctx.webgl;
                    if (webgl === undefined)
                        throw new Error('createUnitGaussianDensityVolume requires `webgl` object in VisualContext');
                    p = tslib_1.__assign(tslib_1.__assign({}, props), { useGpu: true });
                    oldTexture = directVolume ? directVolume.gridTexture.ref.value : undefined;
                    return [4 /*yield*/, gaussian_1.computeUnitGaussianDensityTexture(structure, unit, p, webgl, oldTexture).runInContext(runtime)];
                case 1:
                    densityTextureData = _a.sent();
                    transform = densityTextureData.transform, texture = densityTextureData.texture, bbox = densityTextureData.bbox, gridDim = densityTextureData.gridDim;
                    stats = { min: 0, max: 1, mean: 0.04, sigma: 0.01 };
                    unitToCartn = linear_algebra_1.Mat4.mul(linear_algebra_1.Mat4(), transform, linear_algebra_1.Mat4.fromScaling(linear_algebra_1.Mat4(), gridDim));
                    cellDim = linear_algebra_1.Mat4.getScaling(linear_algebra_1.Vec3(), transform);
                    vol = direct_volume_1.DirectVolume.create(bbox, gridDim, transform, unitToCartn, cellDim, texture, stats, true, directVolume);
                    sphere = geometry_1.Sphere3D.expand(geometry_1.Sphere3D(), unit.boundary.sphere, props.radiusOffset + common_1.getUnitExtraRadius(unit));
                    vol.setBoundingSphere(sphere);
                    return [2 /*return*/, vol];
            }
        });
    });
}
exports.UnitsGaussianDensityVolumeParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, units_visual_1.UnitsDirectVolumeParams), gaussian_1.GaussianDensityTextureParams), { ignoreHydrogens: param_definition_1.ParamDefinition.Boolean(false) });
function UnitsGaussianDensityVolumeVisual(materialId) {
    return units_visual_1.UnitsDirectVolumeVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.UnitsGaussianDensityVolumeParams),
        createGeometry: createUnitsGaussianDensityVolume,
        createLocationIterator: element_1.ElementIterator.fromGroup,
        getLoci: element_1.getElementLoci,
        eachLocation: element_1.eachElement,
        setUpdateState: function (state, newProps, currentProps) {
            if (newProps.resolution !== currentProps.resolution)
                state.createGeometry = true;
            if (newProps.radiusOffset !== currentProps.radiusOffset)
                state.createGeometry = true;
            if (newProps.smoothness !== currentProps.smoothness)
                state.createGeometry = true;
            if (newProps.ignoreHydrogens !== currentProps.ignoreHydrogens)
                state.createGeometry = true;
            if (newProps.traceOnly !== currentProps.traceOnly)
                state.createGeometry = true;
            if (newProps.includeParent !== currentProps.includeParent)
                state.createGeometry = true;
        }
    }, materialId);
}
exports.UnitsGaussianDensityVolumeVisual = UnitsGaussianDensityVolumeVisual;
//# sourceMappingURL=gaussian-density-volume.js.map