"use strict";
/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IsosurfaceRepresentationProvider = exports.IsosurfaceRepresentation = exports.getIsosurfaceParams = exports.IsosurfaceParams = exports.IsosurfaceWireframeVisual = exports.IsosurfaceWireframeParams = exports.createVolumeIsosurfaceWireframe = exports.IsosurfaceMeshVisual = exports.IsosurfaceMeshParams = exports.createVolumeIsosurfaceMesh = exports.eachIsosurface = exports.VolumeIsosurfaceParams = void 0;
var tslib_1 = require("tslib");
var param_definition_1 = require("../../mol-util/param-definition");
var volume_1 = require("../../mol-model/volume");
var mesh_1 = require("../../mol-geo/geometry/mesh/mesh");
var algorithm_1 = require("../../mol-geo/util/marching-cubes/algorithm");
var representation_1 = require("./representation");
var location_iterator_1 = require("../../mol-geo/util/location-iterator");
var location_1 = require("../../mol-model/location");
var lines_1 = require("../../mol-geo/geometry/lines/lines");
var representation_2 = require("../representation");
var loci_1 = require("../../mol-model/loci");
var int_1 = require("../../mol-data/int");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var array_1 = require("../../mol-util/array");
var util_1 = require("./util");
exports.VolumeIsosurfaceParams = {
    isoValue: volume_1.Volume.IsoValueParam
};
function getLoci(volume, props) {
    return volume_1.Volume.Isosurface.Loci(volume, props.isoValue);
}
function getIsosurfaceLoci(pickingId, volume, props, id) {
    var objectId = pickingId.objectId, groupId = pickingId.groupId;
    if (id === objectId) {
        return volume_1.Volume.Cell.Loci(volume, int_1.Interval.ofSingleton(groupId));
    }
    return loci_1.EmptyLoci;
}
function eachIsosurface(loci, volume, props, apply) {
    return util_1.eachVolumeLoci(loci, volume, props.isoValue, apply);
}
exports.eachIsosurface = eachIsosurface;
//
function createVolumeIsosurfaceMesh(ctx, volume, theme, props, mesh) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var ids, surface, transform;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    ctx.runtime.update({ message: 'Marching cubes...' });
                    ids = array_1.fillSerial(new Int32Array(volume.grid.cells.data.length));
                    return [4 /*yield*/, algorithm_1.computeMarchingCubesMesh({
                            isoLevel: volume_1.Volume.IsoValue.toAbsolute(props.isoValue, volume.grid.stats).absoluteValue,
                            scalarField: volume.grid.cells,
                            idField: linear_algebra_1.Tensor.create(volume.grid.cells.space, linear_algebra_1.Tensor.Data1(ids))
                        }, mesh).runAsChild(ctx.runtime)];
                case 1:
                    surface = _a.sent();
                    transform = volume_1.Grid.getGridToCartesianTransform(volume.grid);
                    mesh_1.Mesh.transform(surface, transform);
                    if (ctx.webgl && !ctx.webgl.isWebGL2)
                        mesh_1.Mesh.uniformTriangleGroup(surface);
                    surface.setBoundingSphere(volume_1.Volume.getBoundingSphere(volume));
                    return [2 /*return*/, surface];
            }
        });
    });
}
exports.createVolumeIsosurfaceMesh = createVolumeIsosurfaceMesh;
exports.IsosurfaceMeshParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, mesh_1.Mesh.Params), { quality: tslib_1.__assign(tslib_1.__assign({}, mesh_1.Mesh.Params.quality), { isEssential: false }) }), exports.VolumeIsosurfaceParams);
function IsosurfaceMeshVisual(materialId) {
    return representation_1.VolumeVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.IsosurfaceMeshParams),
        createGeometry: createVolumeIsosurfaceMesh,
        createLocationIterator: function (volume) { return location_iterator_1.LocationIterator(volume.grid.cells.data.length, 1, 1, function () { return location_1.NullLocation; }); },
        getLoci: getIsosurfaceLoci,
        eachLocation: eachIsosurface,
        setUpdateState: function (state, volume, newProps, currentProps) {
            if (!volume_1.Volume.IsoValue.areSame(newProps.isoValue, currentProps.isoValue, volume.grid.stats))
                state.createGeometry = true;
        },
        geometryUtils: mesh_1.Mesh.Utils
    }, materialId);
}
exports.IsosurfaceMeshVisual = IsosurfaceMeshVisual;
//
function createVolumeIsosurfaceWireframe(ctx, volume, theme, props, lines) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var ids, wireframe, transform;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    ctx.runtime.update({ message: 'Marching cubes...' });
                    ids = array_1.fillSerial(new Int32Array(volume.grid.cells.data.length));
                    return [4 /*yield*/, algorithm_1.computeMarchingCubesLines({
                            isoLevel: volume_1.Volume.IsoValue.toAbsolute(props.isoValue, volume.grid.stats).absoluteValue,
                            scalarField: volume.grid.cells,
                            idField: linear_algebra_1.Tensor.create(volume.grid.cells.space, linear_algebra_1.Tensor.Data1(ids))
                        }, lines).runAsChild(ctx.runtime)];
                case 1:
                    wireframe = _a.sent();
                    transform = volume_1.Grid.getGridToCartesianTransform(volume.grid);
                    lines_1.Lines.transform(wireframe, transform);
                    wireframe.setBoundingSphere(volume_1.Volume.getBoundingSphere(volume));
                    return [2 /*return*/, wireframe];
            }
        });
    });
}
exports.createVolumeIsosurfaceWireframe = createVolumeIsosurfaceWireframe;
exports.IsosurfaceWireframeParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, lines_1.Lines.Params), { quality: tslib_1.__assign(tslib_1.__assign({}, lines_1.Lines.Params.quality), { isEssential: false }), sizeFactor: param_definition_1.ParamDefinition.Numeric(1.5, { min: 0, max: 10, step: 0.1 }) }), exports.VolumeIsosurfaceParams);
function IsosurfaceWireframeVisual(materialId) {
    return representation_1.VolumeVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.IsosurfaceWireframeParams),
        createGeometry: createVolumeIsosurfaceWireframe,
        createLocationIterator: function (volume) { return location_iterator_1.LocationIterator(volume.grid.cells.data.length, 1, 1, function () { return location_1.NullLocation; }); },
        getLoci: getIsosurfaceLoci,
        eachLocation: eachIsosurface,
        setUpdateState: function (state, volume, newProps, currentProps) {
            if (!volume_1.Volume.IsoValue.areSame(newProps.isoValue, currentProps.isoValue, volume.grid.stats))
                state.createGeometry = true;
        },
        geometryUtils: lines_1.Lines.Utils
    }, materialId);
}
exports.IsosurfaceWireframeVisual = IsosurfaceWireframeVisual;
//
var IsosurfaceVisuals = {
    'solid': function (ctx, getParams) { return representation_1.VolumeRepresentation('Isosurface mesh', ctx, getParams, IsosurfaceMeshVisual, getLoci); },
    'wireframe': function (ctx, getParams) { return representation_1.VolumeRepresentation('Isosurface wireframe', ctx, getParams, IsosurfaceWireframeVisual, getLoci); },
};
exports.IsosurfaceParams = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, exports.IsosurfaceMeshParams), exports.IsosurfaceWireframeParams), { visuals: param_definition_1.ParamDefinition.MultiSelect(['solid'], param_definition_1.ParamDefinition.objectToOptions(IsosurfaceVisuals)) });
function getIsosurfaceParams(ctx, volume) {
    var p = param_definition_1.ParamDefinition.clone(exports.IsosurfaceParams);
    p.isoValue = volume_1.Volume.createIsoValueParam(volume_1.Volume.IsoValue.relative(2), volume.grid.stats);
    return p;
}
exports.getIsosurfaceParams = getIsosurfaceParams;
function IsosurfaceRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Isosurface', ctx, getParams, representation_2.Representation.StateBuilder, IsosurfaceVisuals);
}
exports.IsosurfaceRepresentation = IsosurfaceRepresentation;
exports.IsosurfaceRepresentationProvider = representation_1.VolumeRepresentationProvider({
    name: 'isosurface',
    label: 'Isosurface',
    description: 'Displays a triangulated isosurface of volumetric data.',
    factory: IsosurfaceRepresentation,
    getParams: getIsosurfaceParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.IsosurfaceParams),
    defaultColorTheme: { name: 'uniform' },
    defaultSizeTheme: { name: 'uniform' },
    isApplicable: function (volume) { return !volume_1.Volume.isEmpty(volume); }
});
//# sourceMappingURL=isosurface.js.map