/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { RepresentationRegistry, Representation, RepresentationProvider } from '../representation';
import { Volume } from '../../mol-model/volume';
export declare class VolumeRepresentationRegistry extends RepresentationRegistry<Volume, Representation.State> {
    constructor();
}
export declare namespace VolumeRepresentationRegistry {
    export const BuiltIn: {
        isosurface: import("./representation").VolumeRepresentationProvider<{
            visuals: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"solid" | "wireframe", string[]>;
            isoValue: import("../../mol-util/param-definition").ParamDefinition.Conditioned<Volume.IsoValue, import("../../mol-util/param-definition").ParamDefinition.Base<Volume.IsoValue>, {
                absolute: import("../../mol-util/param-definition").ParamDefinition.Converted<Readonly<{
                    kind: "absolute";
                    absoluteValue: number;
                }>, number>;
                relative: import("../../mol-util/param-definition").ParamDefinition.Converted<Readonly<{
                    kind: "relative";
                    relativeValue: number;
                }>, number>;
            }>;
            quality: {
                isEssential: boolean;
                type: "select";
                options: readonly (readonly ["custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest", string] | readonly ["custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest", string, string | undefined])[];
                cycle?: boolean | undefined;
                isOptional?: boolean | undefined;
                defaultValue: "custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest";
                label?: string | undefined;
                description?: string | undefined;
                legend?: import("../../mol-util/legend").TableLegend | import("../../mol-util/legend").ScaleLegend | undefined;
                fieldLabels?: {
                    [name: string]: string;
                } | undefined;
                isHidden?: boolean | undefined;
                shortLabel?: boolean | undefined;
                twoColumns?: boolean | undefined;
                category?: string | undefined;
                hideIf?: ((currentGroup: any) => boolean) | undefined;
                help?: ((value: any) => {
                    description?: string | undefined;
                    legend?: import("../../mol-util/legend").TableLegend | import("../../mol-util/legend").ScaleLegend | undefined;
                }) | undefined;
            };
            sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
            lineSizeAttenuation: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
            doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            ignoreLight: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            xrayShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        }, "isosurface">;
        slice: import("./representation").VolumeRepresentationProvider<{
            quality: {
                isEssential: boolean;
                type: "select";
                options: readonly (readonly ["custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest", string] | readonly ["custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest", string, string | undefined])[];
                cycle?: boolean | undefined;
                isOptional?: boolean | undefined;
                defaultValue: "custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest";
                label?: string | undefined;
                description?: string | undefined;
                legend?: import("../../mol-util/legend").TableLegend | import("../../mol-util/legend").ScaleLegend | undefined;
                fieldLabels?: {
                    [name: string]: string;
                } | undefined;
                isHidden?: boolean | undefined;
                shortLabel?: boolean | undefined;
                twoColumns?: boolean | undefined;
                category?: string | undefined;
                hideIf?: ((currentGroup: any) => boolean) | undefined;
                help?: ((value: any) => {
                    description?: string | undefined;
                    legend?: import("../../mol-util/legend").TableLegend | import("../../mol-util/legend").ScaleLegend | undefined;
                }) | undefined;
            };
            dimension: import("../../mol-util/param-definition").ParamDefinition.Mapped<import("../../mol-util/param-definition").ParamDefinition.NamedParams<number, "x"> | import("../../mol-util/param-definition").ParamDefinition.NamedParams<number, "y"> | import("../../mol-util/param-definition").ParamDefinition.NamedParams<number, "z">>;
            isoValue: import("../../mol-util/param-definition").ParamDefinition.Conditioned<Volume.IsoValue, import("../../mol-util/param-definition").ParamDefinition.Base<Volume.IsoValue>, {
                absolute: import("../../mol-util/param-definition").ParamDefinition.Converted<Readonly<{
                    kind: "absolute";
                    absoluteValue: number;
                }>, number>;
                relative: import("../../mol-util/param-definition").ParamDefinition.Converted<Readonly<{
                    kind: "relative";
                    relativeValue: number;
                }>, number>;
            }>;
            interpolation: import("../../mol-util/param-definition").ParamDefinition.Select<"nearest" | "catmulrom" | "mitchell" | "bspline">;
            alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        }, "slice">;
        'direct-volume': import("./representation").VolumeRepresentationProvider<{
            quality: {
                isEssential: boolean;
                type: "select";
                options: readonly (readonly ["custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest", string] | readonly ["custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest", string, string | undefined])[];
                cycle?: boolean | undefined;
                isOptional?: boolean | undefined;
                defaultValue: "custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest";
                label?: string | undefined;
                description?: string | undefined;
                legend?: import("../../mol-util/legend").TableLegend | import("../../mol-util/legend").ScaleLegend | undefined;
                fieldLabels?: {
                    [name: string]: string;
                } | undefined;
                isHidden?: boolean | undefined;
                shortLabel?: boolean | undefined;
                twoColumns?: boolean | undefined;
                category?: string | undefined;
                hideIf?: ((currentGroup: any) => boolean) | undefined;
                help?: ((value: any) => {
                    description?: string | undefined;
                    legend?: import("../../mol-util/legend").TableLegend | import("../../mol-util/legend").ScaleLegend | undefined;
                }) | undefined;
            };
            doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            ignoreLight: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            xrayShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
            renderMode: import("../../mol-util/param-definition").ParamDefinition.Mapped<import("../../mol-util/param-definition").ParamDefinition.NamedParams<import("../../mol-util/param-definition").ParamDefinition.Normalize<{
                isoValue: Volume.IsoValue;
                singleLayer: boolean;
            }>, "isosurface"> | import("../../mol-util/param-definition").ParamDefinition.NamedParams<import("../../mol-util/param-definition").ParamDefinition.Normalize<{
                controlPoints: import("../../mol-math/linear-algebra/3d/vec2").default[];
                list: {
                    kind: "interpolate" | "set";
                    colors: import("../../mol-util/color/color").ColorListEntry[];
                };
            }>, "volume">>;
            stepsPerCell: import("../../mol-util/param-definition").ParamDefinition.Numeric;
            jumpLength: import("../../mol-util/param-definition").ParamDefinition.Numeric;
            alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        }, "direct-volume">;
    };
    type _BuiltIn = typeof BuiltIn;
    export type BuiltIn = keyof _BuiltIn;
    export type BuiltInParams<T extends BuiltIn> = Partial<RepresentationProvider.ParamValues<_BuiltIn[T]>>;
    export {};
}
