"use strict";
/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.eachVolumeLoci = void 0;
var volume_1 = require("../../mol-model/volume");
var int_1 = require("../../mol-data/int");
var common_1 = require("../../mol-math/linear-algebra/3d/common");
function eachVolumeLoci(loci, volume, isoValue, apply) {
    var changed = false;
    if (volume_1.Volume.isLoci(loci)) {
        if (!volume_1.Volume.areEquivalent(loci.volume, volume))
            return false;
        if (apply(int_1.Interval.ofLength(volume.grid.cells.data.length)))
            changed = true;
    }
    else if (volume_1.Volume.Isosurface.isLoci(loci)) {
        if (!volume_1.Volume.areEquivalent(loci.volume, volume))
            return false;
        if (isoValue) {
            if (volume_1.Volume.IsoValue.areSame(loci.isoValue, isoValue, volume.grid.stats))
                return false;
            if (apply(int_1.Interval.ofLength(volume.grid.cells.data.length)))
                changed = true;
        }
        else {
            // TODO find a cheaper way?
            var _a = volume.grid, stats = _a.stats, data = _a.cells.data;
            var eps = stats.sigma;
            var v = volume_1.Volume.IsoValue.toAbsolute(loci.isoValue, stats).absoluteValue;
            for (var i = 0, il = data.length; i < il; ++i) {
                if (common_1.equalEps(v, data[i], eps)) {
                    if (apply(int_1.Interval.ofSingleton(i)))
                        changed = true;
                }
            }
        }
    }
    else if (volume_1.Volume.Cell.isLoci(loci)) {
        if (!volume_1.Volume.areEquivalent(loci.volume, volume))
            return false;
        if (int_1.Interval.is(loci.indices)) {
            if (apply(loci.indices))
                changed = true;
        }
        else {
            int_1.OrderedSet.forEach(loci.indices, function (v) {
                if (apply(int_1.Interval.ofSingleton(v)))
                    changed = true;
            });
        }
    }
    return changed;
}
exports.eachVolumeLoci = eachVolumeLoci;
//# sourceMappingURL=util.js.map