"use strict";
/**
 * Copyright (c) 2018 Mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolMap = exports.SymbolList = exports.TTargs = exports.Types = void 0;
var tslib_1 = require("tslib");
var type_1 = tslib_1.__importDefault(require("../type"));
var symbol_1 = require("../symbol");
var helpers_1 = require("../helpers");
var Types;
(function (Types) {
    Types.AnyVar = type_1.default.Variable('a', type_1.default.Any);
    Types.AnyValueVar = type_1.default.Variable('a', type_1.default.Any);
    Types.ConstrainedVar = type_1.default.Variable('a', type_1.default.Any, true);
    Types.Regex = type_1.default.Value('Core', 'Regex');
    Types.Set = function (t) { return type_1.default.Container('Core', 'Set', t || Types.AnyValueVar); };
    Types.List = function (t) { return type_1.default.Container('Core', 'List', t || Types.AnyVar); };
    Types.Fn = function (t, alias) { return type_1.default.Container('Core', 'Fn', t || Types.AnyVar, alias); };
    Types.Flags = function (t, alias) { return type_1.default.Container('Core', 'Flags', t, alias); };
    Types.BitFlags = Types.Flags(type_1.default.Num, 'BitFlags');
})(Types = exports.Types || (exports.Types = {}));
function unaryOp(type, description) {
    return helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(type) }), type, description);
}
function binOp(type, description) {
    return helpers_1.symbol(symbol_1.Arguments.List(type, { nonEmpty: true }), type, description);
}
function binRel(src, target, description) {
    return helpers_1.symbol(symbol_1.Arguments.Dictionary({
        0: symbol_1.Argument(src),
        1: symbol_1.Argument(src)
    }), target, description);
}
exports.TTargs = symbol_1.Arguments.Dictionary({
    0: symbol_1.Argument(type_1.default.Num),
    1: symbol_1.Argument(type_1.default.Num)
});
var XX = { test: symbol_1.Argument(type_1.default.Str) };
var t = 0;
t.test;
var type = {
    '@header': 'Types',
    bool: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(type_1.default.AnyValue) }), type_1.default.Bool, 'Convert a value to boolean.'),
    num: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(type_1.default.AnyValue) }), type_1.default.Num, 'Convert a value to number.'),
    str: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(type_1.default.AnyValue) }), type_1.default.Str, 'Convert a value to string.'),
    regex: helpers_1.symbol(symbol_1.Arguments.Dictionary({
        0: symbol_1.Argument(type_1.default.Str, { description: 'Expression' }),
        1: symbol_1.Argument(type_1.default.Str, { isOptional: true, description: "Flags, e.g. 'i' for ignore case" })
    }), Types.Regex, 'Creates a regular expression from a string using the ECMAscript syntax.'),
    list: helpers_1.symbol(symbol_1.Arguments.List(Types.AnyVar), Types.List()),
    set: helpers_1.symbol(symbol_1.Arguments.List(Types.AnyValueVar), Types.Set()),
    bitflags: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(type_1.default.Num) }), Types.BitFlags, 'Interpret a number as bitflags.'),
    compositeKey: helpers_1.symbol(symbol_1.Arguments.List(type_1.default.AnyValue), type_1.default.AnyValue),
};
var logic = {
    '@header': 'Logic',
    not: unaryOp(type_1.default.Bool),
    and: binOp(type_1.default.Bool),
    or: binOp(type_1.default.Bool),
};
var ctrl = {
    '@header': 'Control',
    eval: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(Types.Fn(Types.AnyVar)) }), Types.AnyVar, 'Evaluate a function.'),
    fn: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(Types.AnyVar) }), Types.Fn(Types.AnyVar), 'Wrap an expression to a "lazy" function.'),
    if: helpers_1.symbol(symbol_1.Arguments.Dictionary({
        0: symbol_1.Argument(type_1.default.Bool, { description: 'Condition' }),
        1: symbol_1.Argument(type_1.default.Variable('a', type_1.default.Any), { description: 'If true' }),
        2: symbol_1.Argument(type_1.default.Variable('b', type_1.default.Any), { description: 'If false' })
    }), type_1.default.Union([type_1.default.Variable('a', type_1.default.Any), type_1.default.Variable('b', type_1.default.Any)])),
    assoc: helpers_1.symbol(symbol_1.Arguments.Dictionary({
        0: symbol_1.Argument(type_1.default.Str, { description: 'Name' }),
        1: symbol_1.Argument(type_1.default.Variable('a', type_1.default.Any), { description: 'Value to assign' })
    }), type_1.default.Variable('a', type_1.default.Any))
};
var rel = {
    '@header': 'Relational',
    eq: binRel(type_1.default.Variable('a', type_1.default.AnyValue, true), type_1.default.Bool),
    neq: binRel(type_1.default.Variable('a', type_1.default.AnyValue, true), type_1.default.Bool),
    lt: binRel(type_1.default.Num, type_1.default.Bool),
    lte: binRel(type_1.default.Num, type_1.default.Bool),
    gr: binRel(type_1.default.Num, type_1.default.Bool),
    gre: binRel(type_1.default.Num, type_1.default.Bool),
    inRange: helpers_1.symbol(symbol_1.Arguments.Dictionary({
        0: symbol_1.Argument(type_1.default.Num, { description: 'Value to test' }),
        1: symbol_1.Argument(type_1.default.Num, { description: 'Minimum value' }),
        2: symbol_1.Argument(type_1.default.Num, { description: 'Maximum value' })
    }), type_1.default.Bool, 'Check if the value of the 1st argument is >= 2nd and <= 3rd.'),
};
var math = {
    '@header': 'Math',
    add: binOp(type_1.default.Num),
    sub: binOp(type_1.default.Num),
    mult: binOp(type_1.default.Num),
    div: binRel(type_1.default.Num, type_1.default.Num),
    pow: binRel(type_1.default.Num, type_1.default.Num),
    mod: binRel(type_1.default.Num, type_1.default.Num),
    min: binOp(type_1.default.Num),
    max: binOp(type_1.default.Num),
    floor: unaryOp(type_1.default.Num),
    ceil: unaryOp(type_1.default.Num),
    roundInt: unaryOp(type_1.default.Num),
    abs: unaryOp(type_1.default.Num),
    sqrt: unaryOp(type_1.default.Num),
    cbrt: unaryOp(type_1.default.Num),
    sin: unaryOp(type_1.default.Num),
    cos: unaryOp(type_1.default.Num),
    tan: unaryOp(type_1.default.Num),
    asin: unaryOp(type_1.default.Num),
    acos: unaryOp(type_1.default.Num),
    atan: unaryOp(type_1.default.Num),
    sinh: unaryOp(type_1.default.Num),
    cosh: unaryOp(type_1.default.Num),
    tanh: unaryOp(type_1.default.Num),
    exp: unaryOp(type_1.default.Num),
    log: unaryOp(type_1.default.Num),
    log10: unaryOp(type_1.default.Num),
    atan2: binRel(type_1.default.Num, type_1.default.Num)
};
var str = {
    '@header': 'Strings',
    concat: binOp(type_1.default.Str),
    match: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(Types.Regex), 1: symbol_1.Argument(type_1.default.Str) }), type_1.default.Bool)
};
var list = {
    '@header': 'Lists',
    getAt: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(Types.List()), 1: symbol_1.Argument(type_1.default.Num) }), Types.AnyVar),
    equal: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(Types.List()), 1: symbol_1.Argument(Types.List()) }), type_1.default.Bool)
};
var set = {
    '@header': 'Sets',
    has: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(Types.Set(Types.ConstrainedVar)), 1: symbol_1.Argument(Types.ConstrainedVar) }), type_1.default.Bool, 'Check if the the 1st argument includes the value of the 2nd.'),
    isSubset: helpers_1.symbol(symbol_1.Arguments.Dictionary({ 0: symbol_1.Argument(Types.Set(Types.ConstrainedVar)), 1: symbol_1.Argument(Types.Set(Types.ConstrainedVar)) }), type_1.default.Bool, 'Check if the the 1st argument is a subset of the 2nd.')
};
var flags = {
    '@header': 'Flags',
    hasAny: helpers_1.symbol(symbol_1.Arguments.Dictionary({
        0: symbol_1.Argument(Types.Flags(Types.ConstrainedVar)),
        1: symbol_1.Argument(Types.Flags(Types.ConstrainedVar))
    }), type_1.default.Bool, 'Check if the the 1st argument has at least one of the 2nd one\'s flags.'),
    hasAll: helpers_1.symbol(symbol_1.Arguments.Dictionary({
        0: symbol_1.Argument(Types.Flags(Types.ConstrainedVar)),
        1: symbol_1.Argument(Types.Flags(Types.ConstrainedVar))
    }), type_1.default.Bool, 'Check if the the 1st argument has all 2nd one\'s flags.'),
};
var table = {
    '@header': 'Language Primitives',
    type: type,
    logic: logic,
    ctrl: ctrl,
    rel: rel,
    math: math,
    str: str,
    list: list,
    set: set,
    flags: flags
};
helpers_1.normalizeTable(table);
exports.SymbolList = helpers_1.symbolList(table);
exports.SymbolMap = (function () {
    var map = Object.create(null);
    for (var _i = 0, SymbolList_1 = exports.SymbolList; _i < SymbolList_1.length; _i++) {
        var s = SymbolList_1[_i];
        map[s.id] = s;
    }
    return map;
})();
exports.default = table;
//# sourceMappingURL=core.js.map