/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { StructureElement, Structure } from '../mol-model/structure';
import { Script } from '../mol-script/script';
export { Transparency };
declare type Transparency = {
    readonly layers: ReadonlyArray<Transparency.Layer>;
};
declare function Transparency(layers: ReadonlyArray<Transparency.Layer>): Transparency;
declare namespace Transparency {
    type Layer = {
        readonly loci: StructureElement.Loci;
        readonly value: number;
    };
    const Empty: Transparency;
    type Variant = 'single' | 'multi';
    function areEqual(tA: Transparency, tB: Transparency): boolean;
    function isEmpty(transparency: Transparency): boolean;
    function remap(transparency: Transparency, structure: Structure): {
        layers: Layer[];
    };
    function merge(transparency: Transparency): Transparency;
    function filter(transparency: Transparency, filter: Structure): Transparency;
    type ScriptLayer = {
        script: Script;
        value: number;
    };
    function ofScript(scriptLayers: ScriptLayer[], structure: Structure): Transparency;
    type BundleLayer = {
        bundle: StructureElement.Bundle;
        value: number;
    };
    function ofBundle(bundleLayers: BundleLayer[], structure: Structure): Transparency;
    function toBundle(transparency: Transparency): {
        layers: BundleLayer[];
    };
}
