/**
 * Copyright (c) 2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { Structure } from '../../../mol-model/structure';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
import { RepresentationContext, RepresentationParamsGetter } from '../../../mol-repr/representation';
import { StructureRepresentation, StructureRepresentationProvider } from '../../../mol-repr/structure/representation';
import { UnitsVisual } from '../../../mol-repr/structure/units-visual';
import { ValidationReport } from './prop';
import { ComplexVisual } from '../../../mol-repr/structure/complex-visual';
export declare const IntraUnitClashParams: {
    linkCap: PD.BooleanParam;
    sizeFactor: PD.Numeric;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    radialSegments: PD.Numeric;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
};
export declare type IntraUnitClashParams = typeof IntraUnitClashParams;
export declare function IntraUnitClashVisual(materialId: number): UnitsVisual<IntraUnitClashParams>;
export declare const InterUnitClashParams: {
    linkCap: PD.BooleanParam;
    sizeFactor: PD.Numeric;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    radialSegments: PD.Numeric;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
};
export declare type InterUnitClashParams = typeof InterUnitClashParams;
export declare function InterUnitClashVisual(materialId: number): ComplexVisual<InterUnitClashParams>;
export declare const ClashesParams: {
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    visuals: PD.MultiSelect<"intra-clash" | "inter-clash", string[]>;
    linkCap: PD.BooleanParam;
    sizeFactor: PD.Numeric;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
};
export declare type ClashesParams = typeof ClashesParams;
export declare function getClashesParams(ctx: ThemeRegistryContext, structure: Structure): {
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    visuals: PD.MultiSelect<"intra-clash" | "inter-clash", string[]>;
    linkCap: PD.BooleanParam;
    sizeFactor: PD.Numeric;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
};
export declare type ClashesRepresentation = StructureRepresentation<ClashesParams>;
export declare function ClashesRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<Structure, ClashesParams>): ClashesRepresentation;
export declare const ClashesRepresentationProvider: StructureRepresentationProvider<{
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    visuals: PD.MultiSelect<"intra-clash" | "inter-clash", string[]>;
    linkCap: PD.BooleanParam;
    sizeFactor: PD.Numeric;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
}, ValidationReport.Tag.Clashes>;
