/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ValueCell } from '../../mol-util';
import { TextureImage } from '../../mol-gl/renderable/util';
import { Color } from '../../mol-util/color';
import { Vec2, Vec3 } from '../../mol-math/linear-algebra';
import { LocationIterator } from '../util/location-iterator';
import { LocationColor, ColorTheme } from '../../mol-theme/color';
export declare type ColorType = 'uniform' | 'instance' | 'group' | 'groupInstance' | 'vertex' | 'vertexInstance';
export declare type ColorData = {
    uColor: ValueCell<Vec3>;
    tColor: ValueCell<TextureImage<Uint8Array>>;
    uColorTexDim: ValueCell<Vec2>;
    dColorType: ValueCell<string>;
};
export declare function createColors(locationIt: LocationIterator, positionIt: LocationIterator, colorTheme: ColorTheme<any>, colorData?: ColorData): ColorData;
export declare function createValueColor(value: Color, colorData?: ColorData): ColorData;
/** Creates color uniform */
export declare function createUniformColor(locationIt: LocationIterator, color: LocationColor, colorData?: ColorData): ColorData;
export declare function createTextureColor(colors: TextureImage<Uint8Array>, type: ColorType, colorData?: ColorData): ColorData;
/** Creates color texture with color for each instance */
export declare function createInstanceColor(locationIt: LocationIterator, color: LocationColor, colorData?: ColorData): ColorData;
/** Creates color texture with color for each group (i.e. shared across instances) */
export declare function createGroupColor(locationIt: LocationIterator, color: LocationColor, colorData?: ColorData): ColorData;
/** Creates color texture with color for each group in each instance */
export declare function createGroupInstanceColor(locationIt: LocationIterator, color: LocationColor, colorData?: ColorData): ColorData;
/** Creates color texture with color for each vertex (i.e. shared across instances) */
export declare function createVertexColor(locationIt: LocationIterator, color: LocationColor, colorData?: ColorData): ColorData;
/** Creates color texture with color for each vertex in each instance */
export declare function createVertexInstanceColor(locationIt: LocationIterator, color: LocationColor, colorData?: ColorData): ColorData;
