/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __assign, __awaiter, __generator } from "tslib";
import { Task } from '../../mol-task';
import { GaussianDensityCPU } from './gaussian-density/cpu';
import { GaussianDensityTexture2d, GaussianDensityTexture3d } from './gaussian-density/gpu';
// import { GaussianDensityGPU, GaussianDensityTexture } from './gaussian-density/gpu';
var GaussianDensityGPU = typeof document !== 'undefined'
    ? require('./gaussian-density/gpu').GaussianDensityGPU
    : void 0;
var GaussianDensityTexture = typeof document !== 'undefined'
    ? require('./gaussian-density/gpu').GaussianDensityTexture
    : void 0;
export var DefaultGaussianDensityGPUProps = {
    resolution: 1,
    radiusOffset: 0,
    smoothness: 1.5,
};
export var DefaultGaussianDensityProps = __assign(__assign({}, DefaultGaussianDensityGPUProps), { useGpu: true });
export function computeGaussianDensity(position, box, radius, props, webgl) {
    var _this = this;
    return Task.create('Gaussian Density', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, GaussianDensity(ctx, position, box, radius, props, webgl)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    }); });
}
export function GaussianDensity(ctx, position, box, radius, props, webgl) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!props.useGpu) return [3 /*break*/, 1];
                    if (!GaussianDensityGPU)
                        throw 'GPU computation not supported on this platform';
                    if (!webgl)
                        throw 'No WebGL context provided';
                    return [2 /*return*/, GaussianDensityGPU(position, box, radius, props, webgl)];
                case 1: return [4 /*yield*/, GaussianDensityCPU(ctx, position, box, radius, props)];
                case 2: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
export function computeGaussianDensityTexture(position, box, radius, props, webgl, texture) {
    return _computeGaussianDensityTexture(webgl.isWebGL2 ? '3d' : '2d', position, box, radius, props, webgl, texture);
}
export function computeGaussianDensityTexture2d(position, box, radius, props, webgl, texture) {
    return _computeGaussianDensityTexture('2d', position, box, radius, props, webgl, texture);
}
export function computeGaussianDensityTexture3d(position, box, radius, props, webgl, texture) {
    return _computeGaussianDensityTexture('2d', position, box, radius, props, webgl, texture);
}
function _computeGaussianDensityTexture(type, position, box, radius, props, webgl, texture) {
    var _this = this;
    if (!GaussianDensityTexture)
        throw 'GPU computation not supported on this platform';
    return Task.create('Gaussian Density', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, type === '2d' ?
                    GaussianDensityTexture2d(webgl, position, box, radius, false, props, texture) :
                    GaussianDensityTexture3d(webgl, position, box, radius, props, texture)];
        });
    }); });
}
//# sourceMappingURL=gaussian-density.js.map