/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __awaiter, __generator } from "tslib";
import { Task } from '../../mol-task';
import { Column, Table } from '../../mol-data/db';
import { EntityBuilder } from './common/entity';
import { fillSerial } from '../../mol-util/array';
import { BasicSchema, createBasic } from './basic/schema';
import { createModels } from './basic/parser';
function getBasic(table) {
    var entityIds = new Array(table._rowCount);
    var entityBuilder = new EntityBuilder();
    var seqIdStarts = table.position.toArray({ array: Uint32Array });
    var seqIdEnds = new Uint32Array(table._rowCount);
    var stride = seqIdStarts[1] - seqIdStarts[0];
    var objectRadius = stride / 3500;
    for (var i = 0, il = table._rowCount; i < il; ++i) {
        var chr = table.chromosome.value(i);
        var entityId = entityBuilder.getEntityId(chr, 7 /* DNA */, chr);
        entityIds[i] = entityId;
        seqIdEnds[i] = seqIdStarts[i] + stride - 1;
    }
    var ihm_sphere_obj_site = Table.ofPartialColumns(BasicSchema.ihm_sphere_obj_site, {
        id: Column.ofIntArray(fillSerial(new Uint32Array(table._rowCount))),
        entity_id: Column.ofStringArray(entityIds),
        seq_id_begin: Column.ofIntArray(seqIdStarts),
        seq_id_end: Column.ofIntArray(seqIdEnds),
        asym_id: table.chromosome,
        Cartn_x: Column.ofFloatArray(Column.mapToArray(table.x, function (x) { return x * 10; }, Float32Array)),
        Cartn_y: Column.ofFloatArray(Column.mapToArray(table.y, function (y) { return y * 10; }, Float32Array)),
        Cartn_z: Column.ofFloatArray(Column.mapToArray(table.z, function (z) { return z * 10; }, Float32Array)),
        object_radius: Column.ofConst(objectRadius, table._rowCount, Column.Schema.float),
        rmsf: Column.ofConst(0, table._rowCount, Column.Schema.float),
        model_id: Column.ofConst(1, table._rowCount, Column.Schema.int),
    }, table._rowCount);
    return createBasic({
        entity: entityBuilder.getEntityTable(),
        ihm_model_list: Table.ofPartialColumns(BasicSchema.ihm_model_list, {
            model_id: Column.ofIntArray([1]),
            model_name: Column.ofStringArray(['3DG Model']),
        }, 1),
        ihm_sphere_obj_site: ihm_sphere_obj_site
    });
}
//
export { Format3dg };
var Format3dg;
(function (Format3dg) {
    function is(x) {
        return x.kind === '3dg';
    }
    Format3dg.is = is;
    function from3dg(file3dg) {
        return { kind: '3dg', name: '3DG', data: file3dg };
    }
    Format3dg.from3dg = from3dg;
})(Format3dg || (Format3dg = {}));
export function trajectoryFrom3DG(file3dg) {
    var _this = this;
    return Task.create('Parse 3DG', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
        var format, basic;
        return __generator(this, function (_a) {
            format = Format3dg.from3dg(file3dg);
            basic = getBasic(file3dg.table);
            return [2 /*return*/, createModels(basic, format, ctx)];
        });
    }); });
}
//# sourceMappingURL=3dg.js.map