/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { StructureRepresentation, StructureRepresentationProvider } from '../representation';
import { RepresentationParamsGetter, RepresentationContext } from '../../../mol-repr/representation';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
import { Structure } from '../../../mol-model/structure';
export declare const BallAndStickParams: {
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    sizeFactor: PD.Numeric;
    sizeAspectRatio: PD.Numeric;
    visuals: PD.MultiSelect<"element-sphere" | "intra-bond" | "inter-bond", string[]>;
    includeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    excludeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    linkCap: PD.BooleanParam;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    detail: PD.Numeric;
    traceOnly: PD.BooleanParam;
};
export declare type BallAndStickParams = typeof BallAndStickParams;
export declare function getBallAndStickParams(ctx: ThemeRegistryContext, structure: Structure): {
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    sizeFactor: PD.Numeric;
    sizeAspectRatio: PD.Numeric;
    visuals: PD.MultiSelect<"element-sphere" | "intra-bond" | "inter-bond", string[]>;
    includeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    excludeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    linkCap: PD.BooleanParam;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    detail: PD.Numeric;
    traceOnly: PD.BooleanParam;
};
export declare type BallAndStickRepresentation = StructureRepresentation<BallAndStickParams>;
export declare function BallAndStickRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<Structure, BallAndStickParams>): BallAndStickRepresentation;
export declare const BallAndStickRepresentationProvider: StructureRepresentationProvider<{
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    sizeFactor: PD.Numeric;
    sizeAspectRatio: PD.Numeric;
    visuals: PD.MultiSelect<"element-sphere" | "intra-bond" | "inter-bond", string[]>;
    includeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    excludeTypes: PD.MultiSelect<"covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide", ("covalent" | "computed" | "aromatic" | "metal-coordination" | "hydrogen-bond" | "disulfide")[]>;
    ignoreHydrogens: PD.BooleanParam;
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    linkCap: PD.BooleanParam;
    radialSegments: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    detail: PD.Numeric;
    traceOnly: PD.BooleanParam;
}, "ball-and-stick">;
