/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { StructureRepresentation, StructureRepresentationProvider } from '../representation';
import { RepresentationParamsGetter, RepresentationContext } from '../../../mol-repr/representation';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
import { Structure } from '../../../mol-model/structure';
export declare const SpacefillParams: {
    sizeFactor: PD.Numeric;
    detail: PD.Numeric;
    ignoreHydrogens: PD.BooleanParam;
    traceOnly: PD.BooleanParam;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    doubleSided: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type SpacefillParams = typeof SpacefillParams;
export declare function getSpacefillParams(ctx: ThemeRegistryContext, structure: Structure): {
    sizeFactor: PD.Numeric;
    detail: PD.Numeric;
    ignoreHydrogens: PD.BooleanParam;
    traceOnly: PD.BooleanParam;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    doubleSided: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type SpacefillRepresentation = StructureRepresentation<SpacefillParams>;
export declare function SpacefillRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<Structure, SpacefillParams>): SpacefillRepresentation;
export declare const SpacefillRepresentationProvider: StructureRepresentationProvider<{
    sizeFactor: PD.Numeric;
    detail: PD.Numeric;
    ignoreHydrogens: PD.BooleanParam;
    traceOnly: PD.BooleanParam;
    unitKinds: PD.MultiSelect<"spheres" | "atomic" | "gaussians", ("spheres" | "atomic" | "gaussians")[]>;
    doubleSided: PD.BooleanParam;
    ignoreLight: PD.BooleanParam;
    xrayShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    quality: PD.Select<"custom" | "auto" | "highest" | "higher" | "high" | "medium" | "low" | "lower" | "lowest">;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
}, "spacefill">;
