/**
 * Copyright (c) 2018-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Unit, Structure } from '../../../../mol-model/structure';
import { Task } from '../../../../mol-task';
import { ParamDefinition as PD } from '../../../../mol-util/param-definition';
import { Texture } from '../../../../mol-gl/webgl/texture';
import { WebGLContext } from '../../../../mol-gl/webgl/context';
export declare const GaussianDensityParams: {
    useGpu: PD.BooleanParam;
    ignoreHydrogens: PD.BooleanParam;
    traceOnly: PD.BooleanParam;
    includeParent: PD.BooleanParam;
    resolution: PD.Numeric;
    radiusOffset: PD.Numeric;
    smoothness: PD.Numeric;
};
export declare const DefaultGaussianDensityProps: PD.Values<{
    useGpu: PD.BooleanParam;
    ignoreHydrogens: PD.BooleanParam;
    traceOnly: PD.BooleanParam;
    includeParent: PD.BooleanParam;
    resolution: PD.Numeric;
    radiusOffset: PD.Numeric;
    smoothness: PD.Numeric;
}>;
export declare type GaussianDensityProps = typeof DefaultGaussianDensityProps;
export declare const GaussianDensityTextureParams: {
    ignoreHydrogens: PD.BooleanParam;
    traceOnly: PD.BooleanParam;
    includeParent: PD.BooleanParam;
    resolution: PD.Numeric;
    radiusOffset: PD.Numeric;
    smoothness: PD.Numeric;
};
export declare const DefaultGaussianDensityTextureProps: PD.Values<{
    ignoreHydrogens: PD.BooleanParam;
    traceOnly: PD.BooleanParam;
    includeParent: PD.BooleanParam;
    resolution: PD.Numeric;
    radiusOffset: PD.Numeric;
    smoothness: PD.Numeric;
}>;
export declare type GaussianDensityTextureProps = typeof DefaultGaussianDensityTextureProps;
export declare function computeUnitGaussianDensity(structure: Structure, unit: Unit, props: GaussianDensityProps, webgl?: WebGLContext): Task<import("../../../../mol-math/geometry/gaussian-density").GaussianDensityData>;
export declare function computeUnitGaussianDensityTexture(structure: Structure, unit: Unit, props: GaussianDensityTextureProps, webgl: WebGLContext, texture?: Texture): Task<import("../../../../mol-math/geometry/gaussian-density").GaussianDensityTextureData>;
export declare function computeUnitGaussianDensityTexture2d(structure: Structure, unit: Unit, powerOfTwo: boolean, props: GaussianDensityTextureProps, webgl: WebGLContext, texture?: Texture): Task<import("../../../../mol-math/geometry/gaussian-density").GaussianDensityTextureData>;
export declare function computeStructureGaussianDensity(structure: Structure, props: GaussianDensityProps, webgl?: WebGLContext): Task<import("../../../../mol-math/geometry/gaussian-density").GaussianDensityData>;
export declare function computeStructureGaussianDensityTexture(structure: Structure, props: GaussianDensityTextureProps, webgl: WebGLContext, texture?: Texture): Task<import("../../../../mol-math/geometry/gaussian-density").GaussianDensityTextureData>;
export declare function computeStructureGaussianDensityTexture2d(structure: Structure, powerOfTwo: boolean, props: GaussianDensityTextureProps, webgl: WebGLContext, texture?: Texture): Task<import("../../../../mol-math/geometry/gaussian-density").GaussianDensityTextureData>;
