"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = "\nprecision highp float;\nprecision highp sampler2D;\n\nuniform sampler2D tActiveVoxelsPyramid;\nuniform sampler2D tActiveVoxelsBase;\nuniform sampler2D tVolumeData;\nuniform sampler2D tTriIndices;\n\nuniform float uIsoValue;\nuniform float uLevels;\nuniform float uSize;\nuniform float uCount;\n\nuniform vec3 uGridDim;\nuniform vec3 uGridTexDim;\nuniform mat4 uGridTransform;\n\n// scale to volume data coord\nuniform vec2 uScale;\n\n#include common\n\n// cube corners\nconst vec3 c0 = vec3(0., 0., 0.);\nconst vec3 c1 = vec3(1., 0., 0.);\nconst vec3 c2 = vec3(1., 1., 0.);\nconst vec3 c3 = vec3(0., 1., 0.);\nconst vec3 c4 = vec3(0., 0., 1.);\nconst vec3 c5 = vec3(1., 0., 1.);\nconst vec3 c6 = vec3(1., 1., 1.);\nconst vec3 c7 = vec3(0., 1., 1.);\n\nconst float EPS = 0.00001;\n\nvec3 index3dFrom2d(vec2 coord) {\n    vec2 gridTexPos = coord * uGridTexDim.xy;\n    vec2 columnRow = floor(gridTexPos / uGridDim.xy);\n    vec2 posXY = gridTexPos - columnRow * uGridDim.xy;\n    float posZ = columnRow.y * floor(uGridTexDim.x / uGridDim.x) + columnRow.x;\n    vec3 posXYZ = vec3(posXY, posZ);\n    return posXYZ;\n}\n\nvec4 texture3dFrom2dNearest(sampler2D tex, vec3 pos, vec3 gridDim, vec2 texDim) {\n    float zSlice = floor(pos.z * gridDim.z + 0.5); // round to nearest z-slice\n    float column = intMod(zSlice * gridDim.x, texDim.x) / gridDim.x;\n    float row = floor(intDiv(zSlice * gridDim.x, texDim.x));\n    vec2 coord = (vec2(column * gridDim.x, row * gridDim.y) + (pos.xy * gridDim.xy)) / (texDim / uScale);\n    return texture2D(tex, coord + 0.5 / (texDim / uScale));\n}\n\nvec4 voxel(vec3 pos) {\n    return texture3dFrom2dNearest(tVolumeData, pos, uGridDim, uGridTexDim.xy);\n}\n\nvoid main(void) {\n    // get 1D index\n    float vI = dot(floor(uSize * (gl_FragCoord.xy / uSize)), vec2(1.0, uSize));\n\n    // ignore 1D indices outside of the grid\n    if(vI >= uCount) discard;\n\n    float offset = uSize - 2.;\n    float k = 1. / uSize;\n\n    vec2 relativePosition = k * vec2(offset, 0.);\n    vec4 partialSums = texture2D(tActiveVoxelsPyramid, relativePosition);\n    float start = 0.;\n    vec4 starts = vec4(0.);\n    vec4 ends = vec4(0.);\n    float diff = 2.;\n    vec4 m = vec4(0.);\n    vec2 position = vec2(0.);\n    vec4 vI4 = vec4(vI);\n\n    // traverse the different levels of the pyramid\n    for(int i = 1; i < 14; i++) {\n        if(float(i) >= uLevels) break;\n\n        offset -= diff;\n        diff *= 2.;\n        relativePosition = position + k * vec2(offset, 0.);\n\n        ends = partialSums.wzyx + vec4(start);\n        starts = vec4(start, ends.xyz);\n        m = vec4(greaterThanEqual(vI4, starts)) * vec4(lessThan(vI4, ends));\n        relativePosition += m.y * vec2(k, 0.) + m.z * vec2(0., k) + m.w * vec2(k, k);\n\n        start = dot(m, starts);\n        position = 2. * (relativePosition - k * vec2(offset, 0.));\n        partialSums = texture2D(tActiveVoxelsPyramid, relativePosition);\n    }\n\n    ends = partialSums.wzyx + vec4(start);\n    starts = vec4(start, ends.xyz);\n    m = vec4(greaterThanEqual(vI4, starts)) * vec4(lessThan(vI4, ends));\n    position += m.y * vec2(k, 0.) + m.z * vec2(0., k) + m.w * vec2(k, k);\n\n    vec2 coord2d = position / uScale;\n    vec3 coord3d = floor(index3dFrom2d(coord2d) + 0.5);\n\n    float edgeIndex = floor(texture2D(tActiveVoxelsBase, position).a * 255.0 + 0.5);\n\n    // current vertex for the up to 15 MC cases\n    float currentVertex = vI - dot(m, starts);\n\n    // get index into triIndices table\n    float mcIndex = 16. * edgeIndex + currentVertex;\n    vec4 mcData = texture2D(tTriIndices, vec2(intMod(mcIndex, 64.), floor(mcIndex / 64.)) / 64.);\n\n    // bit mask to avoid conditionals (see comment below) for getting MC case corner\n    vec4 m0 = vec4(floor(mcData.a * 255.0 + 0.5));\n\n    // get edge value masks\n    vec4 m1 = vec4(equal(m0, vec4(0., 1., 2., 3.)));\n    vec4 m2 = vec4(equal(m0, vec4(4., 5., 6., 7.)));\n    vec4 m3 = vec4(equal(m0, vec4(8., 9., 10., 11.)));\n\n    // apply bit masks\n    vec3 b0 = coord3d +\n                m1.y * c1 +\n                m1.z * c2 +\n                m1.w * c3 +\n                m2.x * c4 +\n                m2.y * c5 +\n                m2.z * c6 +\n                m2.w * c7 +\n                m3.y * c1 +\n                m3.z * c2 +\n                m3.w * c3;\n    vec3 b1 = coord3d +\n                m1.x * c1 +\n                m1.y * c2 +\n                m1.z * c3 +\n                m2.x * c5 +\n                m2.y * c6 +\n                m2.z * c7 +\n                m2.w * c4 +\n                m3.x * c4 +\n                m3.y * c5 +\n                m3.z * c6 +\n                m3.w * c7;\n\n    // the conditionals that are avoided by above bitmasks\n    // vec3 b0 = coord3d;\n    // vec3 b1 = coord3d;\n    // if (mcIndex == 0.0) {\n    //     b0 += c0; b1 += c1;\n    // } else if (mcIndex == 1.0) {\n    //     b0 += c1; b1 += c2;\n    // } else if (mcIndex == 2.0) {\n    //     b0 += c2; b1 += c3;\n    // } else if (mcIndex == 3.0) {\n    //     b0 += c3; b1 += c0;\n    // } else if (mcIndex == 4.0) {\n    //     b0 += c4; b1 += c5;\n    // } else if (mcIndex == 5.0) {\n    //     b0 += c5; b1 += c6;\n    // } else if (mcIndex == 6.0) {\n    //     b0 += c6; b1 += c7;\n    // } else if (mcIndex == 7.0) {\n    //     b0 += c7; b1 += c4;\n    // } else if (mcIndex == 8.0) {\n    //     b0 += c0; b1 += c4;\n    // } else if (mcIndex == 9.0) {\n    //     b0 += c1; b1 += c5;\n    // } else if (mcIndex == 10.0) {\n    //     b0 += c2; b1 += c6;\n    // } else if (mcIndex == 11.0) {\n    //     b0 += c3; b1 += c7;\n    // }\n    // b0 = floor(b0 + 0.5);\n    // b1 = floor(b1 + 0.5);\n\n    vec4 d0 = voxel(b0 / uGridDim);\n    vec4 d1 = voxel(b1 / uGridDim);\n\n    float v0 = d0.a;\n    float v1 = d1.a;\n\n    float t = (uIsoValue - v0) / (v0 - v1);\n    // t = -0.5;\n    gl_FragData[0].xyz = (uGridTransform * vec4(b0 + t * (b0 - b1), 1.0)).xyz;\n    gl_FragData[0].w = decodeFloatRGB(d0.rgb); // group id\n\n    // normals from gradients\n    vec3 n0 = -normalize(vec3(\n        voxel((b0 - c1) / uGridDim).a - voxel((b0 + c1) / uGridDim).a,\n        voxel((b0 - c3) / uGridDim).a - voxel((b0 + c3) / uGridDim).a,\n        voxel((b0 - c4) / uGridDim).a - voxel((b0 + c4) / uGridDim).a\n    ));\n    vec3 n1 = -normalize(vec3(\n        voxel((b1 - c1) / uGridDim).a - voxel((b1 + c1) / uGridDim).a,\n        voxel((b1 - c3) / uGridDim).a - voxel((b1 + c3) / uGridDim).a,\n        voxel((b1 - c4) / uGridDim).a - voxel((b1 + c4) / uGridDim).a\n    ));\n    gl_FragData[1].xyz = -vec3(\n        n0.x + t * (n0.x - n1.x),\n        n0.y + t * (n0.y - n1.y),\n        n0.z + t * (n0.z - n1.z)\n    );\n\n    mat3 normalMatrix = transpose3(inverse3(mat3(uGridTransform)));\n    gl_FragData[1].xyz = normalMatrix * gl_FragData[1].xyz;\n}\n";
//# sourceMappingURL=isosurface.frag.js.map