declare const _default: "\nprecision highp float;\nprecision highp sampler2D;\n\nuniform sampler2D tTriCount;\nuniform sampler2D tVolumeData;\n\nuniform float uIsoValue;\nuniform vec3 uGridDim;\nuniform vec3 uGridTexDim;\nuniform vec2 uScale;\n\n// cube corners\nconst vec3 c0 = vec3(0., 0., 0.);\nconst vec3 c1 = vec3(1., 0., 0.);\nconst vec3 c2 = vec3(1., 1., 0.);\nconst vec3 c3 = vec3(0., 1., 0.);\nconst vec3 c4 = vec3(0., 0., 1.);\nconst vec3 c5 = vec3(1., 0., 1.);\nconst vec3 c6 = vec3(1., 1., 1.);\nconst vec3 c7 = vec3(0., 1., 1.);\n\nvec3 index3dFrom2d(vec2 coord) {\n    vec2 gridTexPos = coord * uGridTexDim.xy;\n    vec2 columnRow = floor(gridTexPos / uGridDim.xy);\n    vec2 posXY = gridTexPos - columnRow * uGridDim.xy;\n    float posZ = columnRow.y * floor(uGridTexDim.x / uGridDim.x) + columnRow.x;\n    vec3 posXYZ = vec3(posXY, posZ) / uGridDim;\n    return posXYZ;\n}\n\nfloat intDiv(float a, float b) { return float(int(a) / int(b)); }\nfloat intMod(float a, float b) { return a - b * float(int(a) / int(b)); }\n\nvec4 texture3dFrom2dNearest(sampler2D tex, vec3 pos, vec3 gridDim, vec2 texDim) {\n    float zSlice = floor(pos.z * gridDim.z + 0.5); // round to nearest z-slice\n    float column = intMod(zSlice * gridDim.x, texDim.x) / gridDim.x;\n    float row = floor(intDiv(zSlice * gridDim.x, texDim.x));\n    vec2 coord = (vec2(column * gridDim.x, row * gridDim.y) + (pos.xy * gridDim.xy)) / (texDim / uScale);\n    return texture2D(tex, coord);\n}\n\nvec4 voxel(vec3 pos) {\n    return texture3dFrom2dNearest(tVolumeData, pos, uGridDim, uGridTexDim.xy);\n}\n\nvoid main(void) {\n    vec2 uv = gl_FragCoord.xy / uGridTexDim.xy;\n    vec3 posXYZ = index3dFrom2d(uv);\n\n    // get MC case as the sum of corners that are below the given iso level\n    float c = step(voxel(posXYZ).a, uIsoValue)\n        + 2. * step(voxel(posXYZ + c1 / uGridDim).a, uIsoValue)\n        + 4. * step(voxel(posXYZ + c2 / uGridDim).a, uIsoValue)\n        + 8. * step(voxel(posXYZ + c3 / uGridDim).a, uIsoValue)\n        + 16. * step(voxel(posXYZ + c4 / uGridDim).a, uIsoValue)\n        + 32. * step(voxel(posXYZ + c5 / uGridDim).a, uIsoValue)\n        + 64. * step(voxel(posXYZ + c6 / uGridDim).a, uIsoValue)\n        + 128. * step(voxel(posXYZ + c7 / uGridDim).a, uIsoValue);\n    c *= step(c, 254.);\n\n    // get total triangles to generate for calculated MC case from triCount texture\n    float totalTrianglesToGenerate = texture2D(tTriCount, vec2(intMod(c, 16.), floor(c / 16.)) / 16.).a;\n    gl_FragColor = vec4(vec3(totalTrianglesToGenerate * 3.0), c / 255.0);\n\n    // gl_FragColor = vec4(255.0, 0.0, 0.0, voxel(posXYZ + c4 / uGridDim).a * 255.0);\n    // gl_FragColor = vec4(255.0, 0.0, 0.0, voxel(posXYZ).a * 255.0);\n\n    // vec2 uv = vCoordinate;\n    // uv = gl_FragCoord.xy / uGridTexDim.xy;\n\n    // if (uv.y < 0.91) discard;\n    // gl_FragColor = vec4(vCoordinate * 255.0, 0.0, 255.0);\n    // gl_FragColor = vec4(250.0, 0.0, 0.0, 255.0);\n}\n";
export default _default;
