declare const _default: "\nprecision highp float;\nprecision highp int;\nprecision highp sampler2D;\n\nuniform sampler2D tColor;\nuniform sampler2D tPackedDepth;\nuniform vec2 uTexSize;\n\nuniform float uNear;\nuniform float uFar;\nuniform float uFogNear;\nuniform float uFogFar;\nuniform vec3 uFogColor;\n\nuniform float uOcclusionBias;\nuniform float uOcclusionRadius;\n\nuniform float uOutlineScale;\nuniform float uOutlineThreshold;\n\nconst float noiseAmount = 0.0002;\nconst vec4 occlusionColor = vec4(0.0, 0.0, 0.0, 1.0);\n\n#include common\n\nfloat noise(const in vec2 coords) {\n    float a = 12.9898;\n    float b = 78.233;\n    float c = 43758.5453;\n    float dt = dot(coords, vec2(a,b));\n    float sn = mod(dt, 3.14159);\n\n    return fract(sin(sn) * c);\n}\n\nfloat perspectiveDepthToViewZ(const in float invClipZ, const in float near, const in float far) {\n    return (near * far) / ((far - near) * invClipZ - far);\n}\n\nfloat orthographicDepthToViewZ(const in float linearClipZ, const in float near, const in float far) {\n    return linearClipZ * (near - far) - near;\n}\n\nfloat getViewZ(const in float depth) {\n    #if dOrthographic == 1\n        return orthographicDepthToViewZ(depth, uNear, uFar);\n    #else\n        return perspectiveDepthToViewZ(depth, uNear, uFar);\n    #endif\n}\n\nfloat getDepth(const in vec2 coords) {\n    return unpackRGBAToDepth(texture2D(tPackedDepth, coords));\n}\n\nfloat calcSSAO(const in vec2 coords, const in float depth) {\n    float occlusionFactor = 0.0;\n\n    for (int i = -dOcclusionKernelSize; i <= dOcclusionKernelSize; i++) {\n        for (int j = -dOcclusionKernelSize; j <= dOcclusionKernelSize; j++) {\n            vec2 coordsDelta = coords + uOcclusionRadius / float(dOcclusionKernelSize) * vec2(float(i) / uTexSize.x, float(j) / uTexSize.y);\n            coordsDelta += noiseAmount * (noise(coordsDelta) - 0.5) / uTexSize;\n            coordsDelta = clamp(coordsDelta, 0.5 / uTexSize, 1.0 - 1.0 / uTexSize);\n            if (getDepth(coordsDelta) < depth) occlusionFactor += 1.0;\n        }\n    }\n\n    return occlusionFactor / float((2 * dOcclusionKernelSize + 1) * (2 * dOcclusionKernelSize + 1));\n}\n\nvec2 calcEdgeDepth(const in vec2 coords) {\n    vec2 invTexSize = 1.0 / uTexSize;\n    float halfScaleFloor = floor(uOutlineScale * 0.5);\n    float halfScaleCeil = ceil(uOutlineScale * 0.5);\n\n    vec2 bottomLeftUV = coords - invTexSize * halfScaleFloor;\n    vec2 topRightUV = coords + invTexSize * halfScaleCeil;\n    vec2 bottomRightUV = coords + vec2(invTexSize.x * halfScaleCeil, -invTexSize.y * halfScaleFloor);\n    vec2 topLeftUV = coords + vec2(-invTexSize.x * halfScaleFloor, invTexSize.y * halfScaleCeil);\n\n    float depth0 = getDepth(bottomLeftUV);\n    float depth1 = getDepth(topRightUV);\n    float depth2 = getDepth(bottomRightUV);\n    float depth3 = getDepth(topLeftUV);\n\n    float depthFiniteDifference0 = depth1 - depth0;\n    float depthFiniteDifference1 = depth3 - depth2;\n\n    return vec2(\n        sqrt(pow(depthFiniteDifference0, 2.0) + pow(depthFiniteDifference1, 2.0)) * 100.0,\n        min(depth0, min(depth1, min(depth2, depth3)))\n    );\n}\n\nvoid main(void) {\n    vec2 coords = gl_FragCoord.xy / uTexSize;\n    vec4 color = texture2D(tColor, coords);\n\n    #ifdef dOutlineEnable\n        vec2 edgeDepth = calcEdgeDepth(coords);\n        float edgeFlag = step(edgeDepth.x, uOutlineThreshold);\n        color.rgb *= edgeFlag;\n\n        float viewDist = abs(getViewZ(edgeDepth.y));\n        float fogFactor = smoothstep(uFogNear, uFogFar, viewDist) * (1.0 - edgeFlag);\n        color.rgb = mix(color.rgb, uFogColor, fogFactor);\n    #endif\n\n    // occlusion needs to be handled after outline to darken them properly\n    #ifdef dOcclusionEnable\n        float depth = getDepth(coords);\n        if (depth <= 0.99) {\n            float occlusionFactor = calcSSAO(coords, depth);\n            color = mix(color, occlusionColor, uOcclusionBias * occlusionFactor);\n        }\n    #endif\n\n    gl_FragColor = color;\n}\n";
export default _default;
